#include "ExampleTestCase.h"

CPPUNIT_TEST_SUITE_REGISTRATION( ExampleTestCase );


void ExampleTestCase::setUp()
{
  	engine = new BustaEngine();
}

void ExampleTestCase::tearDown()
{
	//delete engine;
}

void ExampleTestCase::testConnection()
{
	char type;
	netGameState ngs;
	netOpponent bgs[4];	
	TCPsocket extra_client;
	IPaddress serverIP;
	
	
	server = new BustaNetServer();
	server->setName("Unit Test");
	server->setMaxClients(4);	
	CPPUNIT_ASSERT( server->init(23427) == 0 );

	CPPUNIT_ASSERT( engine->netInit() == 0 );
	engine->connectIP("127.0.0.1");
	SDLNet_ResolveHost(&serverIP, "127.0.0.1", 23427);
	extra_client = SDLNet_TCP_Open(&serverIP);
	
	CPPUNIT_ASSERT( server->setStateAutoAccept(2) == 1 );

	server->sendUpdateAll();

	CPPUNIT_ASSERT( SDLNet_TCP_Recv(extra_client, &type, 1) >= 0 );
	CPPUNIT_ASSERT( SDLNet_TCP_Recv(extra_client, &ngs, sizeof(netGameState)) >= 0 );
	CPPUNIT_ASSERT( SDLNet_TCP_Recv(extra_client, &bgs, 4*sizeof(netOpponent)) >= 0 );

	CPPUNIT_ASSERT( type == 'A' );
	CPPUNIT_ASSERT( ngs.score == 0 );
	CPPUNIT_ASSERT( bgs[0].health == 0 );
	CPPUNIT_ASSERT( bgs[1].health == 0 );

	delete server;
}

void ExampleTestCase::testMovePieces()
{
	BustaModelFoodPiece* fp;

	engine->getGameState()->getBoard(0,2)->setType(PICKLES);
	fp = engine->getGameState()->getBoard(0,2);

	pushEvent(SDLK_RIGHT);
	pushEvent(SDLK_RIGHT);
	pushEvent(SDLK_k);
	pushEvent(SDLK_RIGHT);
	pushEvent(SDLK_k);
	pushEvent(SDLK_ESCAPE);

	engine->playGame();

	CPPUNIT_ASSERT( engine->getGameState()->getBoard(0, 4)->getType() == PICKLES );
	CPPUNIT_ASSERT( engine->getGameState()->getBoard(0, 4) == fp );
	CPPUNIT_ASSERT( engine->getGameState()->getCursor()->getPosition()->getX() == 0 );
	CPPUNIT_ASSERT( engine->getGameState()->getCursor()->getPosition()->getY() == 3 );
}

int ExampleTestCase::pushEvent(SDLKey sym)
{
	SDL_Event event;

	event.type = SDL_KEYDOWN;
	event.key.state = SDL_PRESSED;
	event.key.keysym.sym = sym;
	event.key.keysym.mod = KMOD_NONE;
	return SDL_PushEvent(&event);
}

void ExampleTestCase::testSwapAndSubmit()
{
	BustaModelFoodPiece* sand[5];
	BustaModelCompleteSandwich* csw;

	pushEvent(SDLK_DOWN);
	pushEvent(SDLK_DOWN);
	pushEvent(SDLK_DOWN);
	pushEvent(SDLK_DOWN);
	pushEvent(SDLK_k);
	pushEvent(SDLK_ESCAPE);

	engine->playGame();	
	
	for(int i = 0; i < 5; i++)
	{
		sand[i] = engine->getGameState()->getBoard(4,i);
	}	

	csw = engine->submit_cus();

	for(int i = 0; i < 5; i++)
	{
		CPPUNIT_ASSERT( sand[i]->getType() == csw->getIngredient(i)->getType() );
	}
}
