#include "ExampleTestCase.h"

CPPUNIT_TEST_SUITE_REGISTRATION( ExampleTestCase );


void ExampleTestCase::setUp()
{
	chef = new BustaModelChef();

	tipjar1 = new BustaModelTipJar(20);
	tipjar2 = new BustaModelTipJar();

	pos = new BustaModelPosition(1,2);
	pos1 = new BustaModelPosition();
	cursor = new BustaModelCursor();

	fp1 = new BustaModelFoodPiece();
	fp2 = new BustaModelFoodPiece(CHEESE,FRESH);

	opp1 = new BustaModelOpponent();
	opp2 = new BustaModelOpponent(1,24,20,"Opponent");
	
	csw = new BustaModelCompleteSandwich();
	
	gs = new BustaModelGameState();

	cust = new BustaModelCustomer();

	opptr[0] = opp1;
	opptr[1] = opp2;

	mpgs = new BustaModelMPGameState(gs,opptr,2);
}

void ExampleTestCase::tearDown()
{
	delete chef;
	delete tipjar1;
	delete tipjar2;
	delete pos;
	delete cursor;
	delete fp1;
	delete fp2;
//	delete opp1;	//deleted in mpgs destructor
//	delete opp2;	//deleted in mpgs destructor
	delete csw;
//	delete gs;		//deleted in mpgs destructor
	delete cust;
	delete mpgs;
}

void ExampleTestCase::testChef()
{
	int health;
	
	health = chef->getHealth();

	CPPUNIT_ASSERT(health == 100);

	chef->setHealth(290);

	CPPUNIT_ASSERT_MESSAGE("Chef setHealth test", chef->getHealth() == 290);
}

void ExampleTestCase::testTipJar()
{
	int tips1, tips2;

	CPPUNIT_ASSERT_MESSAGE("Get size test", tipjar2->getSize() == 20);
	CPPUNIT_ASSERT_MESSAGE("Constructor tests", tipjar2->getSize() == tipjar1->getSize());

	tips1 = tipjar1->getTips();
	tipjar1->addTips(15);
	tips2 = tipjar1->getTips();

	CPPUNIT_ASSERT_MESSAGE("Test gettips", tips1==0);
	CPPUNIT_ASSERT_MESSAGE("Test addtips", tips2==15);
	CPPUNIT_ASSERT_MESSAGE("Test isfull when not full", !tipjar1->isfull());
	

	tipjar1->addTips(13);
	CPPUNIT_ASSERT_MESSAGE("Test overflowing tipjar", tipjar1->getTips()==20);
	CPPUNIT_ASSERT_MESSAGE("Test full tip jar", tipjar1->isfull());

	tips1 = tipjar1->subtractTips(15);
	CPPUNIT_ASSERT_MESSAGE("Test ok subtraction return", tips1==15);
	CPPUNIT_ASSERT_MESSAGE("Test ok subtraction tips", tipjar1->getTips()==5);

	tips2 = tipjar1->subtractTips(10);
	CPPUNIT_ASSERT_MESSAGE("Testing too large subtraction tips", tipjar1->getTips()==0);
	CPPUNIT_ASSERT_MESSAGE("Testing too large subtraction return", tips2==5);

}

void ExampleTestCase::testPosition()
{
	CPPUNIT_ASSERT_MESSAGE("Testing position constructor, getX", pos->getX()==1);
	CPPUNIT_ASSERT_MESSAGE("Testing position constructor, getY", pos->getY()==2);

	CPPUNIT_ASSERT_MESSAGE("Testing default constructor, getX", pos1->getX()==0);
	CPPUNIT_ASSERT_MESSAGE("Testing default constructor, getY", pos1->getY()==0);

	pos->setX(-1);
	pos->setY(54);

	CPPUNIT_ASSERT_MESSAGE("Testing setX", pos->getX()==-1);
	CPPUNIT_ASSERT_MESSAGE("Testing setY", pos->getY()==54);	
}

void ExampleTestCase::testCursor()
{
	int x,y,a;
	BustaModelPosition* pos1;

	x = cursor->getPosition()->getX();
	y = cursor->getPosition()->getY();

	CPPUNIT_ASSERT_MESSAGE("Position x=0",x==0);
	CPPUNIT_ASSERT_MESSAGE("Position y=0",y==0);

	pos1 = cursor->getPosition();

	CPPUNIT_ASSERT_MESSAGE("Position pointers not equal",pos1!=pos);

	cursor->setPosition(1,2);

	pos1 = cursor->getPosition();

	x = cursor->getPosition()->getX();
	y = cursor->getPosition()->getY();

	CPPUNIT_ASSERT_MESSAGE("Position setter x",x==1);
	CPPUNIT_ASSERT_MESSAGE("Position setter y",y==2);

	CPPUNIT_ASSERT_MESSAGE("Get pointer",pos1==cursor->getPosition());	

	cursor->setAlignment(34);
	cursor->setModifier(21);

	CPPUNIT_ASSERT_MESSAGE("Alignment test",cursor->getAlignment()==34);
	CPPUNIT_ASSERT_MESSAGE("Modifier test",cursor->getModifier()==21);
}

void ExampleTestCase::testFoodPiece()
{	
	CPPUNIT_ASSERT_MESSAGE("Testing default contructor type", fp1->getType() == TOMATO);
	CPPUNIT_ASSERT_MESSAGE("Testing default contructor state", fp1->getState() == 0);
	CPPUNIT_ASSERT_MESSAGE("Testing default contructor immobile", fp1->getImmobile() == false);

	CPPUNIT_ASSERT_MESSAGE("Testing user contructor type", fp2->getType() == CHEESE);
	CPPUNIT_ASSERT_MESSAGE("Testing user contructor state", fp2->getState() == 0);
	CPPUNIT_ASSERT_MESSAGE("Testing user contructor immobile", fp2->getImmobile() == false);

	fp2->setState(ROTTING);
	CPPUNIT_ASSERT_MESSAGE("Testing setState() sets properly", fp2->getState() == 2);

	fp1->setImmobile(true);
	CPPUNIT_ASSERT_MESSAGE("Testing setImmobile() sets properly", fp1->getImmobile() == true);

	fp2->setType(ONION);
	CPPUNIT_ASSERT_MESSAGE("Testing getType() sets properly", fp2->getType() == ONION);
}

void ExampleTestCase::testOpponent()
{
	CPPUNIT_ASSERT_MESSAGE("Testing default contructor character", opp1->getCharacter() == 0);
	CPPUNIT_ASSERT_MESSAGE("Testing default contructor health", opp1->getHealth() == 20);
	CPPUNIT_ASSERT_MESSAGE("Testing default contructor tipjar empty", opp1->getTips() == 0);

	CPPUNIT_ASSERT_MESSAGE("Testing user contructor character", opp2->getCharacter() ==1);
	CPPUNIT_ASSERT_MESSAGE("Testing user contructor health", opp2->getHealth() ==24);
	CPPUNIT_ASSERT_MESSAGE("Testing user contructor tipjar empty", opp2->getTips() == 0);
	CPPUNIT_ASSERT_MESSAGE("Testing user constructor name", strcmp(opp2->getName(), (const char*)"Opponent") == 0);
}

void ExampleTestCase::testCompleteSandwich()
{
	for(int i = 0; i < WIDTH; i++)
	{
		CPPUNIT_ASSERT_MESSAGE("Testing default contructor type", csw->getIngredient(i)->getType() == TOMATO);
		CPPUNIT_ASSERT_MESSAGE("Testing default contructor state", csw->getIngredient(i)->getState() == 0);
		CPPUNIT_ASSERT_MESSAGE("Testing default contructor immobile", csw->getIngredient(i)->getImmobile() == false);
	}

	CPPUNIT_ASSERT_MESSAGE("Testing default contructor who", csw->getWho() == 0);
	
	csw->setWho(1);
	
	CPPUNIT_ASSERT_MESSAGE("Testing setWho() who", csw->getWho() == 1);

	csw->setIngredient(new BustaModelFoodPiece(ONION, NORMAL), 1);
	
	CPPUNIT_ASSERT_MESSAGE("Testing setIngredient() type", csw->getIngredient(1)->getType() == ONION);
	CPPUNIT_ASSERT_MESSAGE("Testing setIngredient() state", csw->getIngredient(1)->getState() == 1);
	CPPUNIT_ASSERT_MESSAGE("Testing setIngredient() immobile", csw->getIngredient(1)->getImmobile() == false);
}



void ExampleTestCase::testGameState()
{
	CPPUNIT_ASSERT_MESSAGE("Testing default constructor score", gs->getScore()==0);
	CPPUNIT_ASSERT_MESSAGE("Testing default constructor tipjar", gs->getTipJar()!=NULL);
	CPPUNIT_ASSERT_MESSAGE("Testing default constructor chef", gs->getChef()!=NULL);
	CPPUNIT_ASSERT_MESSAGE("Testing default constructor cursor", gs->getCursor() !=NULL);

	for(int i =0; i<HEIGHT; i++){
		for(int j=0;j<WIDTH; j++){
			CPPUNIT_ASSERT_MESSAGE("Testing default constructor board", gs->getBoard(i,j)!=NULL);
		}
	}

	for(int k = 0; k<NUMCUSTOMERS; k++){
		CPPUNIT_ASSERT_MESSAGE("Testing default constructor customers",gs->getCustomer(k)!=NULL);
	}

	gs->setScore(2000);

	CPPUNIT_ASSERT_MESSAGE("Testing set score", gs->getScore()==2000);

	BustaModelCustomer* c1;
	BustaModelCustomer* c2;
	BustaModelCustomer* c3;
	BustaModelCustomer* c4;

	c1 = gs->getCustomer(0);
	c2 = gs->getCustomer(1);
	c3 = gs->getCustomer(2);
	c4 = new BustaModelCustomer();

	int removed = gs->removeCustomer(c2);

	CPPUNIT_ASSERT_MESSAGE("remove customer return", removed==0);
	CPPUNIT_ASSERT_MESSAGE("remove customer queue 0", gs->getCustomer(0)==c1);
	CPPUNIT_ASSERT_MESSAGE("remove customer queue 1", gs->getCustomer(1)==c3);
	CPPUNIT_ASSERT_MESSAGE("remove customer queue 2", gs->getCustomer(2)==NULL);

	gs->addCustomer(c4);
	CPPUNIT_ASSERT_MESSAGE("add customer", gs->getCustomer(2)==c4);

	//add test for set&remove from board

}

void ExampleTestCase::testCustomer()
{
	CPPUNIT_ASSERT_MESSAGE("constructor time test",cust->getTime()==CUSTOMERTIMER);

	BustaModelFoodPiece** order;

	order = cust->getOrder();

	for(int i=0; i<WIDTH; i++){
		CPPUNIT_ASSERT_MESSAGE("constructor order state test",order[i]->getState()==0);
	}

	int timeleft = cust->getTime();
	timeleft = timeleft+25;

	cust->addTime(25);

	int timeleft2 = cust->getTime();

	CPPUNIT_ASSERT_MESSAGE("add time test",timeleft2==timeleft);
}

void ExampleTestCase::testMPGameState()
{
	BustaModelOpponent* o = mpgs->getOpponent(0);

	CPPUNIT_ASSERT_MESSAGE("MP constructor opp1", o==opp1);
	CPPUNIT_ASSERT_MESSAGE("MP constructor opp2",  mpgs->getOpponent(1)==opp2);
	CPPUNIT_ASSERT_MESSAGE("MP constructor gs",  mpgs->getGameState()==gs);
}