#include "ExampleTestCase.h"

CPPUNIT_TEST_SUITE_REGISTRATION( ExampleTestCase );


void ExampleTestCase::setUp()
{
  	server = new BustaNetServer();
	server->setName("Unit Test");
	server->setMaxClients(4);	
	//server->setDebug(true);

	CPPUNIT_ASSERT( server->init(65535) == 0 );

	SDLNet_ResolveHost(&serverIP, "127.0.0.1", 65535);
	
	client_one = SDLNet_TCP_Open(&serverIP);
	client_two = SDLNet_TCP_Open(&serverIP);

	CPPUNIT_ASSERT( server->setStateAutoAccept(2) == 1 );

	type = 0;
}

void ExampleTestCase::tearDown()
{
	SDLNet_TCP_Close(client_one);
	SDLNet_TCP_Close(client_two);

	delete server;
}

void ExampleTestCase::testSetMaxClients()
{
	server->setMaxClients(8);
	CPPUNIT_ASSERT( server->getMaxClients() == 8 );

	server->setMaxClients(4); //keeps delete in order
}

void ExampleTestCase::testSetName()
{
	server->setName("Busta Server");
	CPPUNIT_ASSERT( strcmp(server->getName(),"Busta Server") == 0 );
}

void ExampleTestCase::testSendAccept()
{
	sc_join_response msg;
	sc_new_connection msg1;

	server->sendAccept(1, true, "Connection Accepted");

	CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_two, &type, 1) >= 0 );
	CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_two, &msg, sizeof(sc_join_response)) >= 0 );

	CPPUNIT_ASSERT( type == 'K' );
	CPPUNIT_ASSERT( msg.clientID == 1 );
	CPPUNIT_ASSERT( strcmp(msg.reason, "Connection Accepted") == 0 );
	CPPUNIT_ASSERT( msg.allowed == true );
	CPPUNIT_ASSERT( msg.settings.maxPlayers == 4 );
	CPPUNIT_ASSERT( strcmp(msg.settings.serverName, "Unit Test") == 0 );

	CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_one, &type, 1) >= 0 );
	CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_one, &msg1, sizeof(sc_new_connection)) >= 0 );
	
	CPPUNIT_ASSERT( type == 'I' );
	CPPUNIT_ASSERT( msg1.playerID == 1 );
	CPPUNIT_ASSERT( strcmp(msg1.playerName, "Default Name") == 0 );


	//server->sendAccept(1, false, "Server Full");

	//CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_two, &type, 1) >= 0 );
	//CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_two, &msg, sizeof(sc_join_response)) >= 0 );

	//CPPUNIT_ASSERT( type == 'K' );
	//CPPUNIT_ASSERT( msg.clientID == -1 );
	//CPPUNIT_ASSERT( strcmp(msg.reason, "Server Full") == 0 );
	//CPPUNIT_ASSERT( msg.allowed == false );
	//CPPUNIT_ASSERT( msg.settings.maxPlayers == 4 );
	//CPPUNIT_ASSERT( strcmp(msg.settings.serverName, "Unit Test") == 0 );
}

void ExampleTestCase::testSendCharacterSelectedAllO()
{
	client c_two;
	c_two.id_number = 1;
	BustaModelCharacter character = C1;
	sc_character_selected msg;

	server->sendCharacterSelectedAllO(&c_two, character);

	CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_one, &type, 1) >= 0 );
	CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_one, &msg, sizeof(sc_character_selected)) >= 0 );
	
	CPPUNIT_ASSERT( type == 'F' );
	CPPUNIT_ASSERT( msg.selectingPlayer == c_two.id_number );
	CPPUNIT_ASSERT( character == msg.character );
}

void ExampleTestCase::testSendCharacterSelectCursorAllO()
{
	client c_two;
	c_two.id_number = 1;
	netPosition pos;
	pos.x1 = 1;
	pos.y1 = 1;
	sc_character_select_cursor msg;

	server->sendCharacterSelectCursorAllO(&c_two, pos);

	CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_one, &type, 1) >= 0 );
	CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_one, &msg, sizeof(sc_character_select_cursor)) >= 0 );
	
	CPPUNIT_ASSERT( type == 'G' );
	CPPUNIT_ASSERT( msg.playerID == c_two.id_number );
	CPPUNIT_ASSERT( pos.x1 == msg.cursorLoc.x1 );
	CPPUNIT_ASSERT( pos.y1 == msg.cursorLoc.y1 );
}

void ExampleTestCase::testSendCharacterSelectModeAll()
{
	server->sendCharacterSelectModeAll();

	CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_one, &type, 1) >= 0 );
	CPPUNIT_ASSERT( type == 'D' );
}

void ExampleTestCase::testSendDisconnectedAllO()
{
	client c_two;
	c_two.id_number = 1;
	sc_player_disconnect msg;

	server->sendDisconnectedAllO(&c_two, "Test Message");

	CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_one, &type, 1) >= 0 );
	CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_one, &msg, sizeof(sc_player_disconnect)) >= 0 );
	
	CPPUNIT_ASSERT( type == 'C' );
	CPPUNIT_ASSERT( msg.playerID == c_two.id_number );
	CPPUNIT_ASSERT( strcmp(msg.discMessage, "Test Message") == 0 );
}

void ExampleTestCase::testSendEndAll()
{
	int rank[4], score[4];
	server->sendEndAll();

	CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_one, &type, 1) >= 0 );
	CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_one, rank, 4*sizeof(int)) >= 0 );
	CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_one, score, 4*sizeof(int)) >= 0 );

	CPPUNIT_ASSERT( type == 'M' );
	CPPUNIT_ASSERT( rank[0] == -1 );
	CPPUNIT_ASSERT( score[0] == -1 );
}

void ExampleTestCase::testSendGameStartAll()
{
	server->sendGameStartAll();

	CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_one, &type, 1) >= 0 );
	CPPUNIT_ASSERT( type == 'B' );
}

void ExampleTestCase::testSendPlayerLostAllO()
{
	client c_two;
	c_two.id_number = 1;
	sc_player_lost msg;

	server->sendPlayerLostAllO(&c_two);

	CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_one, &type, 1) >= 0 );
	CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_one, &msg, sizeof(sc_player_lost)) >= 0 );
	
	CPPUNIT_ASSERT( type == 'E' );
	CPPUNIT_ASSERT( msg.playerID == c_two.id_number );
}

void ExampleTestCase::testSendSandwichAllO()
{
	client c_one, c_two;
	c_one.id_number = 0;
	c_two.id_number = 1;
	SpecialSandwich sandwich = CONFUSE;
	sc_sandwich_send msg;

	server->sendSandwichAllO(&c_one, &c_two, sandwich);

	CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_one, &type, 1) >= 0 );
	CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_one, &msg, sizeof(sc_sandwich_send)) >= 0 );
	
	CPPUNIT_ASSERT( type == 'H' );
	CPPUNIT_ASSERT( msg.originatingPlayer == c_two.id_number );
	CPPUNIT_ASSERT( msg.targetPlayer == c_one.id_number );
	CPPUNIT_ASSERT( msg.sandwich == sandwich );
}

void ExampleTestCase::testSendUpdateAll()
{
	netGameState ngs;
	netOpponent bgs[4];

	server->sendUpdateAll();

	CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_one, &type, 1) >= 0 );
	CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_one, &ngs, sizeof(netGameState)) >= 0 );
	CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_one, &bgs, 4*sizeof(netOpponent)) >= 0 );

	CPPUNIT_ASSERT( type == 'A' );
	CPPUNIT_ASSERT( ngs.score == 0 );
	CPPUNIT_ASSERT( bgs[0].health == 0 );
	CPPUNIT_ASSERT( bgs[1].health == 0 );


	CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_two, &type, 1) >= 0 );
	CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_two, &ngs, sizeof(netGameState)) >= 0 );
	CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_two, &bgs, 4*sizeof(netOpponent)) >= 0 );

	CPPUNIT_ASSERT( type == 'A' );
	CPPUNIT_ASSERT( ngs.score == 0 );
	CPPUNIT_ASSERT( bgs[0].health == 0 );
	CPPUNIT_ASSERT( bgs[1].health == 0 );
}

void ExampleTestCase::testSetStateAccept()
{
	type = 'g';

	//cs_ready messages
	CPPUNIT_ASSERT( SDLNet_TCP_Send(client_one, &type, 1) >= 0 );
	CPPUNIT_ASSERT( SDLNet_TCP_Send(client_two, &type, 1) >= 0 );
	
	CPPUNIT_ASSERT( server->setStateAccept() == 1 );
}

void ExampleTestCase::testSetStateCharSelect()
{
	type = 'c';
	cs_character_cursor msg0;
	msg0.cursorLoc.x1 = 3;
	msg0.cursorLoc.y1 = 4;
	cs_character_select msg1;
	msg1.character = C2;
	cs_character_select msg2;
	msg2.character = C3;
	sc_character_selected msg3;
	sc_character_select_cursor msg4;

	//a fake cursor message
	CPPUNIT_ASSERT( SDLNet_TCP_Send(client_one, &type, 1) >= 0 );
	CPPUNIT_ASSERT( SDLNet_TCP_Send(client_one, &msg0, sizeof(cs_character_cursor)) >= 0 );

	//some fake cs_character_select messages
	type = 'b';
	CPPUNIT_ASSERT( SDLNet_TCP_Send(client_one, &type, 1) >= 0 );
	CPPUNIT_ASSERT( SDLNet_TCP_Send(client_one, &msg1, sizeof(cs_character_select)) >= 0 );
	
	CPPUNIT_ASSERT( SDLNet_TCP_Send(client_two, &type, 1) >= 0 );
	CPPUNIT_ASSERT( SDLNet_TCP_Send(client_two, &msg2, sizeof(cs_character_select)) >= 0 );
	

	CPPUNIT_ASSERT( server->setStateCharSelect() == 1 );


	//check for the cursor movement first, its the oldest after all
	CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_two, &type, 1) >= 0 );
	CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_two, &msg4, sizeof(sc_character_select_cursor)) >= 0 );
	CPPUNIT_ASSERT( type == 'G' );
	CPPUNIT_ASSERT( msg4.playerID == 0 );
	CPPUNIT_ASSERT( msg4.cursorLoc.x1 == msg0.cursorLoc.x1 );
	CPPUNIT_ASSERT( msg4.cursorLoc.y1 == msg0.cursorLoc.y1 );

	//informing each other they've selected a character
	CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_one, &type, 1) >= 0 );
	CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_one, &msg3, sizeof(sc_character_selected)) >= 0 );
	CPPUNIT_ASSERT( type == 'F' );
	CPPUNIT_ASSERT( msg3.selectingPlayer == 1 );
	CPPUNIT_ASSERT( msg3.character == msg2.character );

	CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_two, &type, 1) >= 0 );
	CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_two, &msg3, sizeof(sc_character_selected)) >= 0 );
	CPPUNIT_ASSERT( type == 'F' );
	CPPUNIT_ASSERT( msg3.selectingPlayer == 0 );
	CPPUNIT_ASSERT( msg3.character == msg1.character );
	
	//confirmation to both players that character selection is over
	CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_one, &type, 1) >= 0 );
	CPPUNIT_ASSERT( type == 'B' );

	CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_two, &type, 1) >= 0 );
	CPPUNIT_ASSERT( type == 'B' );
}

void ExampleTestCase::testSetStateGame()
{
	netGameState gs, recv_gs;
	netOpponent recv_opp[4];
	sc_player_lost msg;
	int a[4], b[4];
	cs_opponent_view_select opp_view;
	gs.score = 5;
	opp_view.opponentID = 0;
	

	//type = 'a';  // gamestate update
	//CPPUNIT_ASSERT( SDLNet_TCP_Send(client_one, &type, 1) >= 0 );
	//CPPUNIT_ASSERT( SDLNet_TCP_Send(client_one, &gs, sizeof(netGameState)) >= 0 );

	type = 'f'; // focus opponent
	CPPUNIT_ASSERT( SDLNet_TCP_Send(client_two, &type, 1) >= 0 );
	CPPUNIT_ASSERT( SDLNet_TCP_Send(client_two, &opp_view, sizeof(cs_opponent_view_select)) >= 0 );

	type = 'e'; // player lost
	CPPUNIT_ASSERT( SDLNet_TCP_Send(client_two, &type, 1) >= 0 );


	CPPUNIT_ASSERT( server->setStateGame() == 1 );


	//CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_one, &type, 1) >= 0 );
	//CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_one, &recv_gs, sizeof(netGameState)) >= 0 );
	//CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_one, &recv_opp, 4*sizeof(netOpponent)) >= 0 );

	//CPPUNIT_ASSERT( type == 'A' );
	//CPPUNIT_ASSERT( recv_gs.score == gs.score );

	//CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_two, &type, 1) >= 0 );
	//CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_two, &recv_gs, sizeof(netGameState)) >= 0 );
	//CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_two, &recv_opp, 4*sizeof(netOpponent)) >= 0 );

	//CPPUNIT_ASSERT( type == 'A' );
	//CPPUNIT_ASSERT( recv_gs.score == gs.score );

	CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_one, &type, 1) >= 0 );
	CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_one, &msg, sizeof(sc_player_lost)) >= 0 );

	CPPUNIT_ASSERT( type == 'E' );
	CPPUNIT_ASSERT( msg.playerID == 1 );

	CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_one, &type, 1) >= 0 );
	CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_one, &a, 4*sizeof(int)) >= 0 );
	CPPUNIT_ASSERT( SDLNet_TCP_Recv(client_one, &b, 4*sizeof(int)) >= 0 );

	CPPUNIT_ASSERT( type == 'M' );
	CPPUNIT_ASSERT( a[0] == 1 );
	CPPUNIT_ASSERT( a[1] == 0 );
	CPPUNIT_ASSERT( a[2] == -1 );
	CPPUNIT_ASSERT( a[3] == -1 );

	CPPUNIT_ASSERT( b[0] == 0 );
	CPPUNIT_ASSERT( b[1] == 0 );
	CPPUNIT_ASSERT( b[2] == -1 );
	CPPUNIT_ASSERT( b[3] == -1 );

}