/*
 * BustaGrahics.cpp
 * Busta' Sandwich Graphics Engine Implementation
 * Authors: Julian Chan, Thomas Patecky
 *
 * Created: 6:03 pm,July 15, 2005
 * Last Modified: July 15, 2005
 *
 */

#include "BustaGraphics.h"
#include <stdio.h>
#include <stdlib.h>


BustaGraphics::BustaGraphics()
{
	special_menu_is_set = false;
	myScreen = SDL_SetVideoMode(SCREEN_RES_WIDTH, SCREEN_RES_HEIGHT,32,SDL_SWSURFACE|SDL_RESIZABLE|SDL_HWPALETTE);
	mySprites = new BustaGraphicsSprites();
	myFont = new SFont(mySprites->getBMPFont(STD));
	myChalkFont = new SFont(mySprites->getBMPFont(CHALK));
	initButtonLocs();

		try{
			gcn::Widget::setGlobalFont(gui_font);
			gui_image_loader = new gcn::SDLImageLoader();
			gcn::Image::setImageLoader(gui_image_loader);
			gui_graphics = new gcn::SDLGraphics();
			gui_graphics->setTarget(myScreen);
			gui_font = new gcn::ImageFont(CS_BMP_FONT, CS_BMP_FONT_CHAR_SET);
			gcn::Widget::setGlobalFont(gui_font);
			gui_input=  new gcn::SDLInput();
		}
		catch(gcn::Exception e)
		{
			printf(e.getMessage().c_str());
		}
		

	//Animations
	overall_anim_frame = 0;
	chef_anim_frame = 0;
	customer_anim_frame = 0;

	chefLastTime = 0;
	chefCurrTime = 0;

	crusherState = 2; //0:CrusherOpenFoodInside, 1:CrusherClosed, 2:CrusherOpen
	crusherLastTime = 0;
	crusherCurrTime = 0;
	inCrusher = NULL;
}

BustaGraphics::BustaGraphics(FoodType** special_sandwiches, char** sandwich_names, int num_sandwiches, int num_sandwiches_sp)
{
	
	special_menu_is_set = false;
	myScreen = SDL_SetVideoMode(SCREEN_RES_WIDTH, SCREEN_RES_HEIGHT,32,SDL_SWSURFACE|SDL_RESIZABLE|SDL_HWPALETTE);
	SDL_WM_SetCaption("Busta' Sandwich", "kicks ass");
	mySprites = new BustaGraphicsSprites();
	myFont = new SFont(mySprites->getBMPFont(STD));
	myChalkFont = new SFont(mySprites->getBMPFont(CHALK));
	initButtonLocs();

		try{
			gcn::Widget::setGlobalFont(gui_font);
			gui_image_loader = new gcn::SDLImageLoader();
			gcn::Image::setImageLoader(gui_image_loader);
			gui_graphics = new gcn::SDLGraphics();
			gui_graphics->setTarget(myScreen);
			gui_font = new gcn::ImageFont(CS_BMP_FONT, CS_BMP_FONT_CHAR_SET);
			gcn::Widget::setGlobalFont(gui_font);
			gui_input=  new gcn::SDLInput();
		}
		catch(gcn::Exception e)
		{
			printf(e.getMessage().c_str());
		}
		

	//Animations
	overall_anim_frame = 0;
	chef_anim_frame = 0;
	customer_anim_frame = 0;

	chefLastTime = 0;
	chefCurrTime = 0;

	crusherState = 2; //0:CrusherOpenFoodInside, 1:CrusherClosed, 2:CrusherOpen
	crusherLastTime = 0;
	crusherCurrTime = 0;
	inCrusher = NULL;

	setSpecialSandwiches(special_sandwiches, sandwich_names, num_sandwiches, num_sandwiches_sp);

}

void BustaGraphics::submissionAlert(BustaModelCompleteSandwich* sandwich){
	crusherState = 0;
	crusherCurrTime = SDL_GetTicks();
	crusherLastTime = crusherCurrTime;
	inCrusher = sandwich;
}

void BustaGraphics::setSpecialSandwiches(FoodType** special_sandwiches, char** sandwich_names, int num_sandwiches, int num_spSandwiches)
{		
	num_special_sws = num_sandwiches;
	num_special_sws_sp = num_spSandwiches;

	//Set special sandwiches
	special_sw = new FoodType*[num_sandwiches];
	special_sw_names = new char*[num_sandwiches];
	for(int i=0; i<num_sandwiches; i++)
	{
			special_sw[i] = new FoodType[BOARD_SIZE];

			for (int j=0; j<BOARD_SIZE; j++)
				special_sw[i][j] = special_sandwiches[i][j];

			special_sw_names[i] = sandwich_names[i];
	}
	special_menu_is_set = true;
}
	

/*
BustaGraphics::BustaGraphics(SDL_Surface* screen)
{
myScreen = screen;
	mySprites = new BustaGraphicsSprites();
	myFont = new SFont(mySprites->getBMPFont(0));
	myLogoFont = new SFont(mySprites->getBMPFont());
	initButtonLocs();
}
*/


void BustaGraphics::fillScreen(Uint32 color)
{
		SDL_FillRect(myScreen, NULL, color);
}


void BustaGraphics::drawBoard(int x, int y, BustaModelGameState* gs)
{
		//Draw a big black block under the food pieces
		SDL_Rect rect;
		rect.x = x; rect.y = y;
		rect.w = BOARD_SIZE * (FOOD_PIECE_GAP + FOOD_PIECE_SIZE);
		rect.h = BOARD_SIZE * (FOOD_PIECE_GAP + FOOD_PIECE_SIZE);
		SDL_FillRect(myScreen, &rect, SP_BG_COLOR);
		//Draw the pieces
		for(int row=0; row<BOARD_SIZE; row++)
				for(int col=0; col<BOARD_SIZE; col++)
						drawFoodPiece(gs->getBoard(row, col), 
									x + col * (FOOD_PIECE_GAP + FOOD_PIECE_SIZE),
									y + row * (FOOD_PIECE_GAP + FOOD_PIECE_SIZE));
}

void BustaGraphics::drawCrusher()
{
	SDL_Rect rect;
	rect.x = CRUSHER_X;
	rect.y = CRUSHER_Y;

	crusherCurrTime = SDL_GetTicks();
	if(crusherCurrTime - crusherLastTime > CRUSHER_DELAY){
		if(crusherState < 2)
			crusherState++;
		crusherLastTime = crusherCurrTime;
	}
		
	updateRect(&rect, mySprites->getCrusher(crusherState));

	if(crusherState == 0){
		for(int i=0;i<5;i++){
			drawFoodPiece(inCrusher->getIngredient(i),CRUSHER_FOODX + (FOOD_PIECE_SIZE + FOOD_PIECE_GAP)* i,CRUSHER_FOODY);
		}
	}

}

//Draws one FoodPiece
void BustaGraphics::drawFoodPiece(BustaModelFoodPiece* food_piece, int x, int y)
{
		SDL_Rect rect;

		rect.x = x;
		rect.y = y;

		updateRect(&rect, mySprites->getFoodSprite(food_piece));
}

//Draws one FoodPiece
void BustaGraphics::drawSmallFoodPiece(BustaModelFoodPiece* food_piece, int x, int y)
{
		SDL_Rect rect;

		rect.x = x;
		rect.y = y;

		updateRect(&rect, mySprites->getSmallFoodSprite(food_piece));
}

void BustaGraphics::drawScore(int x, int y, int score)
{
		char score_str[16];
		sprintf(score_str, "%d", score);
		myFont->write(myScreen, score_str, x, y);
}


void BustaGraphics::drawCursor(int board_x, int board_y, int row, int col, int swap_alingment)
{
		SDL_Rect cursor_rect;
		
		//Draw the main cursor
		cursor_rect.x = board_x + col * (FOOD_PIECE_GAP + FOOD_PIECE_SIZE);
		cursor_rect.y = board_y + row * (FOOD_PIECE_GAP + FOOD_PIECE_SIZE);
		updateRect(&cursor_rect, mySprites->getCursorSprites());
		
		//Draw the swap cursor
		switch (swap_alingment)
		{
				case LEFT:	col--; break;
				case RIGHT: col++; break;
				case UP:	row--; break;
				case DOWN:	row++; break;
		}
		cursor_rect.x = board_x + col * (FOOD_PIECE_GAP + FOOD_PIECE_SIZE);
		cursor_rect.y = board_y + row * (FOOD_PIECE_GAP + FOOD_PIECE_SIZE);
		updateRect(&cursor_rect, mySprites->getSwapCursorSprites());
}

void BustaGraphics::drawChef(int x, int y, BustaModelChef* c)
{
		SDL_Rect rect;
		rect.x = x;
		rect.y = y;

		ANIM_CHEF_TYPES state;

		if (c->getHealth() < 10)
				state = CHEF_DYING;
		else if (c->getHealth() < 70)
				state = CHEF_BLOATED;
		else
				state = CHEF_HEALTHY;

		//draw the current frame
		updateRect(&rect, mySprites->getChefSprites(c->getCharacter(), state, chef_anim_frame));
		
		//increment to the next chef animation frame if frame delay has been met
		//Warning: crazy math ahead. It works because C evaluate any int < 1 as zero.
		//chef_anim_frame += (overall_anim_frame % ANIM_CHEF_DELAY) / (ANIM_CHEF_DELAY-1);
		//loop the animation
		//chef_anim_frame %= ANIM_CHEF_NUM_FRAMES; 
		chefCurrTime = SDL_GetTicks();
		if(chef_anim_frame == 0 && chefCurrTime - chefLastTime > ANIM_CHEF_DELAY){
			chefLastTime = chefCurrTime;
			chef_anim_frame++;
		}
		else if(chef_anim_frame != 0 && chefCurrTime - chefLastTime > ANIM_CHEF_FRAMERATE){
			chefLastTime = chefCurrTime;
			chef_anim_frame = (chef_anim_frame+1)%ANIM_CHEF_NUM_FRAMES;
		}
}

/*Uint32 BustaGraphics::updateChefFrame(Uint32 interval, void *param)
{
		chefCurrTime = SDL_GetTicks();
		if(chef_anim_frame == 0 && chefCurrTime - chefLastTime > ANIM_CHEF_DELAY){
			chefLastTime = chefCurrTime;
			chef_anim_frame++;
		}
		else if(chef_anim_frame != 0 && chefCurrTime - chefLastTime > ANIM_CHEF_FRAMERATE){
			chefLastTime = chefCurrTime;
			chef_anim_frame = (chef_anim_frame+1)%ANIM_CHEF_NUM_FRAMES;
		}

		chef_anim_frame++;
		chef_anim_frame %= ANIM_CHEF_NUM_FRAMES;
		if(chef_anim_frame == 0)
			return(ANIM_CHEF_DELAY);
		else
			return(ANIM_CHEF_FRAMERATE);
}*/

//Only draws the content state for now
void BustaGraphics::drawCustomer(int x, int y, BustaModelCustomer* customer)
{
		SDL_Rect rect;
		rect.x = x - 2;
		rect.y = y - 2;
		rect.w = SMALL_FOOD_PIECE_SIZE * BOARD_SIZE + 4;
		rect.h = SMALL_FOOD_PIECE_SIZE + CUSTOMER_HEIGHT + 4;
		SDL_FillRect(myScreen, &rect, SP_BG_COLOR);

		ANIM_CUSTOMER_STATES state;		
		if (customer->getTime() < 10)
				state = CUSTOMER_PISSED;
		else
				state = CUSTOMER_CONTENT;

		//Draws the customer
		rect.x = x; rect.y = y;
		updateRect(&rect, mySprites->getCustomerSprites(state,customer->portrait));

		//** Draw the time left bar
		rect.x = x + CUSTOMER_WIDTH;
		rect.y = y + CUSTOMER_HEIGHT - HEALTH_BAR_HEIGHT - 5;
		rect.h = HEALTH_BAR_HEIGHT;

		//Draw time bar background
		rect.w = CUSTOMER_TIME_BAR_WIDTH;
		SDL_FillRect(myScreen, &rect, HEALTH_BAR_BG_COLOR);
	
		//Draw the time remaining. i.e. the foreground
		rect.w = 0;
		rect.w = CUSTOMER_TIME_BAR_WIDTH * customer->getTime() / customer->getAllottedTime();
		SDL_FillRect(myScreen, &rect, HEALTH_BAR_FG_COLOR);

		//increment to the next chef animation frame if frame delay has been met
		//Warning: crazy math ahead. It works because C evaluate any int < 1 as zero.
		customer_anim_frame += (overall_anim_frame % ANIM_CUSTOMER_DELAY) / (ANIM_CUSTOMER_DELAY - 1);
		//loop the animation
		customer_anim_frame %= CUSTOMER_NUM_FRAMES; 
}

void BustaGraphics::drawSpecialSandwichMenu(int x, int y, bool multiplayer)
{
		SDL_Rect rect;
		BustaModelFoodPiece* fp[BOARD_SIZE];
		for(int i=0; i<BOARD_SIZE; i++)
			fp[i] = new BustaModelFoodPiece();
	

		rect.x = x; rect.y = y;
		updateRect(&rect, mySprites->getChalkboard());
		/*
		rect.w = BOARD_SIZE * (FOOD_PIECE_GAP + FOOD_PIECE_SIZE) + 20;
		rect.h = num_special_sws * SPECIAL_SANDWICH_GAP + 100;
		SDL_FillRect(myScreen, &rect, SP_BG_COLOR);
		*/
	
		x += 60;
		y += 40;
		myChalkFont->write(myScreen, "Special Sandwiches", x+40, y);
		y += 33;

		int sand;
		if(multiplayer)
			sand = num_special_sws;
		else
			sand = num_special_sws_sp;

		for (int sw=0; sw<sand; sw++)
		{
				
				myChalkFont->write(myScreen, special_sw_names[sw], x, y);
				for (int food=0; food<BOARD_SIZE; food++)
				{
						fp[food]->setType(special_sw[sw][food]);	
				}

				drawOrder(x + SS_TEXT_LENGTH, y+22, fp);
				y += SPECIAL_SANDWICH_GAP;
		}
		for(int i=0; i<BOARD_SIZE; i++)
			delete fp[i];

}
void BustaGraphics::drawHealthBar(int x, int y,int health, int max_health)
{
		SDL_Rect rect;
		rect.x = x;
		rect.y = y;
		rect.h = HEALTH_BAR_HEIGHT;

		//Draw the background
		rect.w = HEALTH_BAR_WIDTH;
		SDL_FillRect(myScreen, &rect, HEALTH_BAR_BG_COLOR);
	
		//Draw the "blood" remaining. i.e. the foreground
		rect.w = HEALTH_BAR_WIDTH * health / max_health;
		SDL_FillRect(myScreen, &rect, HEALTH_BAR_FG_COLOR);
}

void BustaGraphics::drawOrder(int x, int y, BustaModelFoodPiece** order_array)
{
		//Draws an overlapping array of food pieces
		for (int i=0; i<ORDER_LENGTH; i++)
			drawSmallFoodPiece(order_array[i], x + (SMALL_FOOD_PIECE_SIZE * i), y);
}


void BustaGraphics::updateRect(SDL_Rect* rect, SDL_Surface* rectSurface)
{
		SDL_BlitSurface(rectSurface, NULL, myScreen, rect);
}

//Single-Player Game

void BustaGraphics::drawTipJar(int x , int y, BustaModelTipJar* tj)
{
		SDL_Rect rect;
		rect.x = x;
		rect.y = y;

		int percent_full = 100 * tj->getTips() / tj->getSize();
		updateRect(&rect, mySprites->getTipJarSprite(percent_full));

}


/*
void BustaGraphics::initMPGameScrn(gcn::ActionListener* al)
{
		initSPGameScrn(al);
}
*/
//Multi-Player Game
void BustaGraphics::initMPGameScrn(gcn::ActionListener* al,BustaModelCharacter* chefID, char** opponent_names, int num_opponents)
{
		//Reuse the GUI stuff that's in SP game
		initSPGameScrn(al);

		//Initialize the opponent buttons
		char filename[MAX_FILE_LENGTH];
		char event_ID[32];
		for(int i=0; i<num_opponents; i++)
		{
				sprintf(filename, "%s/Chefs/Chef%d/small.bmp", PATH_SPRITES, chefID[i]);
				sprintf(event_ID, "Opponent%d", i);
				opponent_buttons[i] = new OpponentButton( opponent_names[i],
												new gcn::Image(filename),
												MP_OPPONENT_BUTTON_WIDTH,
												MP_OPPONENT_BUTTON_HEIGHT);
				opponent_buttons[i]->setEventId(event_ID);
				opponent_buttons[i]->addActionListener(al);
				//Make two columns
				gui_top->add(opponent_buttons[i], MP_OPPONENTS_ALIGN_X + (i%2)*(MP_OPPONENT_BUTTON_WIDTH+5), MP_OPPONENTS_ALIGN_Y + (i/2)*(MP_OPPONENT_BUTTON_HEIGHT+5));
		}
		
		//Submit to opponent buttons
		submit_opponent_button = new gcn::Button(MP_SUBMIT_OPPONENT_BUTTON_ID);
		submit_opponent_button->setEventId(MP_SUBMIT_OPPONENT_BUTTON_ID);
		submit_opponent_button->addActionListener(al);
		submit_opponent_button->setFocusable(false);
		gui_top->add(submit_opponent_button, MP_SUBMIT_OPPONENT_BUTTON_X, MP_SUBMIT_OPPONENT_BUTTON_Y);

		//Show special sandwich menu button
		show_special_button = new gcn::Button(MP_SHOW_SPECIAL_BUTTON_ID);
		show_special_button->setEventId(MP_SHOW_SPECIAL_BUTTON_ID);
		show_special_button->addActionListener(al);
		gui_top->add(show_special_button, MP_SHOW_SPECIAL_BUTTON_X, MP_SHOW_SPECIAL_BUTTON_Y);

		//add animation timers
		chefLastTime = SDL_GetTicks();
		chefCurrTime = SDL_GetTicks();

		crusherState = 2; //0:CrusherOpenFoodInside, 1:CrusherClosed, 2:CrusherOpen
		crusherLastTime = SDL_GetTicks();
		crusherCurrTime = SDL_GetTicks();

}

void BustaGraphics::setOpponents(BustaModelMPGameState* mp_gs, int num_opponents)
{
		for(int i=0; i<num_opponents; i++)
				opponent_buttons[i]->setOpponent(mp_gs->getOpponent(i));
}
void BustaGraphics::drawMPGame(BustaModelGameState* gs, BustaModelMPGameState* mp_gs, bool drawSpecialMenu, int num_opponents)
{

		SDL_Rect rect;
		rect.x = 0; rect.y = 0;
		updateRect(&rect, mySprites->getGameBackground());
	
		drawLocalPlayerGame(gs);

		//Draw a a line separating local and network player
	
		rect.x = SCREEN_RES_WIDTH/2 - 2;
		rect.y = 0;
		rect.w = 4;
		rect.h = SCREEN_RES_HEIGHT;
		SDL_FillRect(myScreen, &rect, HEALTH_BAR_FG_COLOR);

		BustaModelGameState* net_player_gs = mp_gs->getGameState();

		//Draw the game state of the net player
		drawBoard(MP_BOARD_TOP_LEFT_X, MP_BOARD_TOP_LEFT_Y, net_player_gs);
		drawScore(MP_SCORE_POS_X, MP_SCORE_POS_Y, net_player_gs->getScore());
		drawChef(MP_CHEF_TOP_LEFT_POS_X, MP_CHEF_TOP_LEFT_POS_Y, net_player_gs->getChef());

		//Draws the customers and their orders
		for (int i=0; i<NUMCUSTOMERS; i++)
		{
			drawCustomer(MP_CUSTOMER_POS_X,
						 MP_CUSTOMER_POS_Y + (NUMCUSTOMERS-i-1)*(CUSTOMER_HEIGHT+SMALL_FOOD_PIECE_SIZE+10),
						 net_player_gs->getCustomer(i));
			drawOrder(MP_ORDER_POS_X, MP_ORDER_POS_Y + (NUMCUSTOMERS-i-1)*(CUSTOMER_HEIGHT+SMALL_FOOD_PIECE_SIZE+10), net_player_gs->getCustomer(i)->getOrder());
		}

		drawHealthBar(MP_HEALTH_BAR_POS_X, MP_HEALTH_BAR_POS_Y, net_player_gs->getChef()->getHealth(), CHEF_MAX_HEALTH);
		drawTipJar(MP_TIP_JAR_POS_X, MP_TIP_JAR_POS_Y, net_player_gs->getTipJar());
		drawCursor(MP_BOARD_TOP_LEFT_X, 
				   MP_BOARD_TOP_LEFT_Y,
				   net_player_gs->getCursor()->getPosition()->getX(),
				   net_player_gs->getCursor()->getPosition()->getY(),
				   net_player_gs->getCursor()->getAlignment());


		//Draw the special sandwich menu
		if(special_menu_is_set && drawSpecialMenu)
		{
			//Check if the button has been changed to "close special menu" yet.
			if ( show_special_button->getCaption().c_str() == MP_SHOW_SPECIAL_BUTTON_ID)
			{
				
				show_special_button->setCaption(MP_CLOSE_SPECIAL_BUTTON_ID);
				show_special_button->setEventId(MP_CLOSE_SPECIAL_BUTTON_ID);
				show_special_button->adjustSize();
			}

			for(int i=0; i<num_opponents; i++){
				opponent_buttons[i]->setVisible(false);
			}
			
			drawSpecialSandwichMenu(SS_MENU_X, SS_MENU_Y,true);
			
		}
		else
		{
			for(int i=0; i<num_opponents; i++){
				opponent_buttons[i]->setVisible(true);
			}

			if ( show_special_button->getCaption().c_str() == MP_CLOSE_SPECIAL_BUTTON_ID)
			{
				show_special_button->setCaption(MP_SHOW_SPECIAL_BUTTON_ID);
				show_special_button->setEventId(MP_SHOW_SPECIAL_BUTTON_ID);
				show_special_button->adjustSize();
			}
		}

		if(gs->getTipJar()->isfull()){
			submit_opponent_button->setVisible(true);
			submit_self_button->setVisible(true);
		}
		else{
			submit_opponent_button->setVisible(false);
			submit_self_button->setVisible(false);
		}

		
		gui->draw();

		SDL_Flip(myScreen);

		overall_anim_frame++;
}
void BustaGraphics::drawMPGame(BustaModelGameState* gs, BustaModelMPGameState* mp_gs)
{
		SDL_Rect rect;
		rect.x = 0; rect.y = 0;
		updateRect(&rect, mySprites->getGameBackground());
		
		drawLocalPlayerGame(gs);

		//Draw a a line separating local and network player
		rect.x = SCREEN_RES_WIDTH/2 - 2;
		rect.y = 0;
		rect.w = 4;
		rect.h = SCREEN_RES_HEIGHT;
		SDL_FillRect(myScreen, &rect, HEALTH_BAR_FG_COLOR);

		BustaModelGameState* net_player_gs = mp_gs->getGameState();

		//Draw the game state of the net player
		drawBoard(MP_BOARD_TOP_LEFT_X, MP_BOARD_TOP_LEFT_Y, net_player_gs);
		drawScore(MP_SCORE_POS_X, MP_SCORE_POS_Y, net_player_gs->getScore());
		drawChef(MP_CHEF_TOP_LEFT_POS_X, MP_CHEF_TOP_LEFT_POS_Y, net_player_gs->getChef());
		//Draws the customers and their orders
		for (int i=0; i<NUMCUSTOMERS; i++)
		{
			drawCustomer(MP_CUSTOMER_POS_X,
						 MP_CUSTOMER_POS_Y + (NUMCUSTOMERS-i-1)*(CUSTOMER_HEIGHT+SMALL_FOOD_PIECE_SIZE+10),
						 net_player_gs->getCustomer(i));
			drawOrder(MP_ORDER_POS_X, MP_ORDER_POS_Y + (NUMCUSTOMERS-i-1)*(CUSTOMER_HEIGHT+SMALL_FOOD_PIECE_SIZE+10), net_player_gs->getCustomer(i)->getOrder());
		}
		drawHealthBar(MP_HEALTH_BAR_POS_X, MP_HEALTH_BAR_POS_Y, net_player_gs->getChef()->getHealth(), CHEF_MAX_HEALTH);
		drawTipJar(MP_TIP_JAR_POS_X, MP_TIP_JAR_POS_Y, net_player_gs->getTipJar());
		drawCursor(MP_BOARD_TOP_LEFT_X, 
				   MP_BOARD_TOP_LEFT_Y,
				   net_player_gs->getCursor()->getPosition()->getX(),
				   net_player_gs->getCursor()->getPosition()->getY(),
				   net_player_gs->getCursor()->getAlignment());
		
		if(gs->getTipJar()->isfull()){
			submit_opponent_button->setVisible(true);
			submit_self_button->setVisible(true);
		}
		else{
			submit_opponent_button->setVisible(false);
			submit_self_button->setVisible(false);
		}

		gui->draw();

		SDL_Flip(myScreen);

		overall_anim_frame++;
}

void BustaGraphics::killMPGameScrn()
{
		killSPGameScrn();
		for(int i=0; i<MAX_OPPONENTS; i++)
			if (opponent_buttons[i] != NULL)
				delete opponent_buttons[i];
}

void BustaGraphics::drawOpponentBoard(BustaModelGameState gs)
{
		printf("Called drawOpponentBoard\n");
}

/**** Main Screen ************
 * ***************************/
void BustaGraphics::drawMainScreen(MainScrnButton selected)
{
		fillScreen(MS_BG_COLOR_BLUE);

		//Draw Logo
		SDL_Rect rect;
		rect.x = MS_LOGO_POS_X;
		rect.y = MS_LOGO_POS_Y;
		updateRect(&rect, mySprites->getMainScreenLogo());
		
		//Draw User Selections
		myFont->writeCentered(myScreen, "Single Player", bMainScrn[MS_START_SP].box.y);
		myFont->writeCentered(myScreen, "Multi Player", bMainScrn[MS_START_MP].box.y);
		myFont->writeCentered(myScreen, "High Scores", bMainScrn[MS_HIGH_SCORE].box.y);
		myFont->writeCentered(myScreen, "Key Bindings", bMainScrn[MS_OPTIONS].box.y);
		myFont->writeCentered(myScreen, "Quit", bMainScrn[MS_QUIT].box.y);
		
		//Draw a slice of cheese next to the current user selection
		BustaModelFoodPiece* cheese = new BustaModelFoodPiece();
		cheese->setType(CHEESE);
		rect.x = MS_START_SP_POS_X - FOOD_PIECE_SIZE;
		//(FOOD_PIECE_SIZE/2) to center the cheese vertically
		rect.y = MS_START_SP_POS_Y - FOOD_PIECE_SIZE/2 + (MS_START_SP_H * selected) + 15;
		updateRect(&rect, mySprites->getFoodSprite(cheese));

		SDL_Flip(myScreen);
}

/** SP Game Screen *********
 * *************************/
void BustaGraphics::initSPGameScrn(gcn::ActionListener* al)
{

		gui_top = new gcn::Container();
		gui_top->setDimension(gcn::Rectangle(0, 0, SCREEN_RES_WIDTH, SCREEN_RES_WIDTH));
		gui_top->setOpaque(false);
		gui = new gcn::Gui();
		gui->setGraphics(gui_graphics);
		gui->setInput(gui_input);
		gui->setTop(gui_top);

		//Initialize widgets
		submit_self_button = new gcn::Button("Self");
		submit_self_button->setEventId("Self");
		submit_self_button->addActionListener(al);
		submit_self_button->setFocusable(false);
		
		submit_customer_button = new gcn::Button("Customer");
		submit_customer_button->setEventId("Customer");
		submit_customer_button->addActionListener(al);
		submit_customer_button->setFocusable(false);
		
		end_game_button = new gcn::Button("End Game");
		end_game_button->setEventId("End Game");
		end_game_button->addActionListener(al);
		end_game_button->setFocusable(false);

		history_text_box = new gcn::TextBox(HISTORY_INIT_TEXT);
		history_text_box->setEditable(false);
		history_text_box->setOpaque(true);

		gui_top->add(submit_self_button, SELF_BUTTON_X, SELF_BUTTON_Y );
		gui_top->add(submit_customer_button, CUSTOMER_BUTTON_X, CUSTOMER_BUTTON_Y);
		gui_top->add(end_game_button, END_GAME_BUTTON_X, END_GAME_BUTTON_Y);
		gui_top->add(history_text_box, HISTORY_TEXT_BOX_X, HISTORY_TEXT_BOX_Y);

		//add animation timers
		chefLastTime = SDL_GetTicks();
		chefCurrTime = SDL_GetTicks();

		crusherState = 2; //0:CrusherOpenFoodInside, 1:CrusherClosed, 2:CrusherOpen
		crusherLastTime = SDL_GetTicks();
		crusherCurrTime = SDL_GetTicks();

}

void BustaGraphics::addHistory(const char* string)
{
		int newLines;
		int length = 0;

		//Count number of lines this new string will require
		while(string[length] != '\0')
			length++;
		newLines = length/HISTORY_TEXT_LINE_CHARS + 1;

		//Limit what would be displayed if there are too many lines
		if (newLines >= HISTORY_TEXT_LINES)
			newLines = HISTORY_TEXT_LINES;
		
		//"Scroll" the old text up.
		for(int n=newLines+1; n<HISTORY_TEXT_LINES; n++)
			history_text_box->setTextRow(n-newLines, history_text_box->getTextRow(n));
	
		//Write the new string
		int currentChar = 0;
		char line[HISTORY_TEXT_LINE_CHARS];
		for(int n=HISTORY_TEXT_LINES-newLines; n<HISTORY_TEXT_LINES; n++)
		{
			//Break up the string into lines
			for(int i=0; i<=HISTORY_TEXT_LINE_CHARS; i++)
			{
				if (currentChar < length)
					line[i] = string[currentChar];
				else
					line[i] = ' ';

				currentChar++;
			}
			history_text_box->setTextRow(n, line);
		}
}

void BustaGraphics::killSPGameScrn()
{
		delete gui_top;
		delete gui;
		delete submit_self_button;
		delete submit_customer_button;
		delete end_game_button;
		delete history_text_box;

}
void BustaGraphics::drawSPGame(BustaModelGameState* gs)
{
		//fillScreen(SP_BG_COLOR);
		SDL_Rect rect;
		rect.x = 0; rect.y = 0;
		updateRect(&rect, mySprites->getGameBackground());


		drawLocalPlayerGame(gs);
		if(special_menu_is_set)
			drawSpecialSandwichMenu(SS_MENU_X, SS_MENU_Y,false);


		if(gs->getTipJar()->isfull()){
			submit_self_button->setVisible(true);
		}
		else{
			submit_self_button->setVisible(false);
		}

		gui->draw();

		SDL_Flip(myScreen);

		overall_anim_frame++;
}


void BustaGraphics::drawLocalPlayerGame(BustaModelGameState* gs)
{
		drawBoard(BOARD_TOP_LEFT_X, BOARD_TOP_LEFT_Y, gs);
		drawScore(SCORE_POS_X, SCORE_POS_Y, gs->getScore());
		drawChef(CHEF_TOP_LEFT_POS_X, CHEF_TOP_LEFT_POS_Y, gs->getChef());
		
		//Draws the customers and their orders
		for (int i=0; i<NUMCUSTOMERS; i++)
		{
			drawCustomer(CUSTOMER_POS_X,
						 CUSTOMER_POS_Y + (NUMCUSTOMERS-i-1)*(CUSTOMER_HEIGHT+SMALL_FOOD_PIECE_SIZE+10),
						 gs->getCustomer(i));
			drawOrder(ORDER_POS_X, ORDER_POS_Y + (NUMCUSTOMERS-i-1)*(CUSTOMER_HEIGHT+SMALL_FOOD_PIECE_SIZE+10), gs->getCustomer(i)->getOrder());
		}

		drawHealthBar(HEALTH_BAR_POS_X, HEALTH_BAR_POS_Y, gs->getChef()->getHealth(), CHEF_MAX_HEALTH);
		drawTipJar(TIP_JAR_POS_X, TIP_JAR_POS_Y, gs->getTipJar());
		drawCursor(BOARD_TOP_LEFT_X, BOARD_TOP_LEFT_Y, gs->getCursor()->getPosition()->getX(), gs->getCursor()->getPosition()->getY(), gs->getCursor()->getAlignment());

		//draw sandwich crusher

		drawCrusher();
}

void BustaGraphics::newdrawSPGame(BustaModelGameState* gs)
{
		//fillScreen(SP_BG_COLOR);
		SDL_Rect rect;
		rect.x = 0; rect.y = 0;
		updateRect(&rect, mySprites->getGameBackground());

		drawLocalPlayerGame(gs);
		
		if(special_menu_is_set)
			drawSpecialSandwichMenu(SS_MENU_X, SS_MENU_Y,false);

		//Draw the buttons and such
		gui->draw();

		SDL_Flip(myScreen);

		overall_anim_frame++;
}

/**** Connection Screen ******
 *****************************/

void BustaGraphics::initConnectScrn(gcn::ActionListener* al)
{
		
		gui_top = new gcn::Container();
		gui_top->setDimension(gcn::Rectangle(0, 0, SCREEN_RES_WIDTH, SCREEN_RES_HEIGHT));
		gui_top->setOpaque(false);

		gui = new gcn::Gui();
		gui->setGraphics(gui_graphics);
		gui->setInput(gui_input);
		gui->setTop(gui_top);

		printf("kasha!\n");

		//Initialize GUI elements
		connect_scrn_server_ip_label = new gcn::Label("Busta Sandwich Server IP Address:");
		connect_scrn_name_label = new gcn::Label("Player Name:");

		connect_scrn_server_ip_field = new gcn::TextField("                                "); //make it 32-char wide to match the label. guichan won't let you just set the field width
		connect_scrn_server_ip_field->setText(""); //set the content back to an empty string.

		connect_scrn_name_field = new gcn::TextField("                                "); //make it 32-char wide to match the label. guichan won't let you just set the field width
		connect_scrn_name_field->setText(""); //set the content back to an empty string.

		connect_scrn_connect_button = new gcn::Button(CS_CONNECT_BUTTON_ID);
		connect_scrn_connect_button->setEventId(CS_CONNECT_BUTTON_ID);
		connect_scrn_connect_button->addActionListener(al);

		connect_scrn_cancel_button = new gcn::Button(CS_CANCEL_BUTTON_ID);
		connect_scrn_cancel_button->setEventId(CS_CANCEL_BUTTON_ID);
		connect_scrn_cancel_button->addActionListener(al);

		connect_scrn_ready_button = new gcn::Button(CS_READY_BUTTON_ID);
		connect_scrn_ready_button->setEventId(CS_READY_BUTTON_ID);
		connect_scrn_ready_button->addActionListener(al);

		connect_scrn_waiting_label = new gcn::Label("Waiting for other players...");


		gui_top->add(connect_scrn_server_ip_label, CS_IP_FIELD_LABEL_POS_X, CS_IP_FIELD_LABEL_POS_Y);
		gui_top->add(connect_scrn_server_ip_field, CS_SERVER_IP_POS_X, CS_SERVER_IP_POS_Y);
		gui_top->add(connect_scrn_waiting_label, CS_SERVER_IP_POS_X, CS_SERVER_IP_POS_Y+20);
		gui_top->add(connect_scrn_name_label, CS_NAME_FIELD_X, CS_NAME_FIELD_Y);
		gui_top->add(connect_scrn_name_field, CS_NAME_FIELD_X, CS_NAME_FIELD_Y + 20);
		gui_top->add(connect_scrn_connect_button, CS_CONNECT_BUTTON_POS_X, CS_CONNECT_BUTTON_POS_Y);
		gui_top->add(connect_scrn_cancel_button, CS_CANCEL_BUTTON_POS_X, CS_CANCEL_BUTTON_POS_Y);
		gui_top->add(connect_scrn_ready_button, CS_READY_BUTTON_POS_X, CS_READY_BUTTON_POS_Y);
		//gui_top->moveToBottom(connect_scrn_server_ip_field);
}
void BustaGraphics::handleGUIEvent(SDL_Event* event)
{
		gui_input->pushInput(*event);
		gui->logic();
}

void BustaGraphics::drawConnectScrn(ConnectScrnStates state, ConnectScrnPlayerList* list)
{
		//Fill background
		//fillScreen(SP_BG_COLOR);
		SDL_Rect rect;
		rect.x = 0; rect.y = 0;
		updateRect(&rect, mySprites->getGameBackground());

		myFont->write(myScreen, "Multiplayer",CS_BG_BOX_X,CS_BG_BOX_Y-40);
		rect.x = CS_BG_BOX_X; 
		rect.y = CS_BG_BOX_Y;
		rect.w = CS_BG_BOX_WIDTH;
		rect.h = CS_BG_BOX_HEIGHT;
		SDL_FillRect(myScreen, &rect, SP_BG_COLOR);


		//Add/remove/change buttons depending on the different states
		switch (state)
		{
				case CS_CONNECTED:
						connect_scrn_connect_button->setCaption(CS_DISCONNECT_BUTTON_ID);
						connect_scrn_connect_button->setEventId(CS_DISCONNECT_BUTTON_ID);
						connect_scrn_connect_button->adjustSize();
						//Add the "ready" button
						connect_scrn_ready_button->setVisible(true);
						break;
				
				case CS_NOT_CONNECTED:
						if (connect_scrn_connect_button->getCaption().c_str() != CS_CONNECT_BUTTON_ID)
						{
							connect_scrn_connect_button->setCaption(CS_CONNECT_BUTTON_ID);
							connect_scrn_connect_button->setEventId(CS_CONNECT_BUTTON_ID);
							connect_scrn_connect_button->adjustSize();
						}
                        
						connect_scrn_connect_button->setVisible(true);
						connect_scrn_ready_button->setVisible(false);
						connect_scrn_waiting_label->setVisible(false);
						break;
				
				case CS_READY:
						if (connect_scrn_connect_button->getCaption().c_str() != CS_DISCONNECT_BUTTON_ID)
						{
							connect_scrn_connect_button->setCaption(CS_DISCONNECT_BUTTON_ID);
							connect_scrn_connect_button->setEventId(CS_DISCONNECT_BUTTON_ID);
							connect_scrn_connect_button->adjustSize();
						}
						connect_scrn_ready_button->setVisible(false);
						connect_scrn_connect_button->setVisible(true);
						connect_scrn_waiting_label->setVisible(true);
						break;

		}
	
		gui->draw();

		myFont->write(myScreen, "Connected Players", CS_PLAYER_LIST_ALIGN_X, CS_PLAYER_LIST_ALIGN_Y - CS_PLAYER_LIST_GAP - 10);
		//Draws a line
		rect.x = CS_PLAYER_LIST_ALIGN_X;
		rect.y = CS_PLAYER_LIST_ALIGN_Y - 10;
		rect.w = CS_LINE_LENGTH;
		rect.h = CS_LINE_THICKNESS;
		SDL_FillRect(myScreen, &rect, HEALTH_BAR_BG_COLOR);

		//Draw a list of connected players
		CS_STATUS_LIGHT light_color;
		int pos_y = CS_PLAYER_LIST_ALIGN_Y;
		rect.x = CS_PLAYER_LIST_ALIGN_X - 30;
		
		while(list)
		{
				char info[32];
				sprintf(info, "%s", list->name);
				myFont->write(myScreen, info, CS_PLAYER_LIST_ALIGN_X, pos_y);

				if (list->isReady)
						light_color = GREEN_LIGHT;
				else
						light_color = RED_LIGHT;

				rect.y = pos_y + 8;
				updateRect(&rect, mySprites->getConnectScrnStatusLight(light_color) );

				//Update drawing coordinate for the next player
				pos_y += CS_PLAYER_LIST_GAP;
				list = list->next;
		}

		SDL_Flip(myScreen);		
}

const char* BustaGraphics::getConnectScrnTextField()
{
		if (connect_scrn_name_field != NULL)
				return connect_scrn_name_field->getText().c_str();	
		else
				return "";
}

const char* BustaGraphics::getServerIPAddress()
{
		if (connect_scrn_server_ip_field != NULL)
				return connect_scrn_server_ip_field->getText().c_str();	
		else
				return "";
}

const char* BustaGraphics::getPlayerName()
{
		if (connect_scrn_server_ip_field != NULL)
				return connect_scrn_server_ip_field->getText().c_str();	
		else
				return "";
}

void BustaGraphics::killConnectScrn()
{
		delete gui_top;
		delete gui;
		delete connect_scrn_server_ip_label;
		delete connect_scrn_server_ip_field;
		delete connect_scrn_name_label;
		delete connect_scrn_name_field;
		delete connect_scrn_connect_button;
		delete connect_scrn_cancel_button;
		delete connect_scrn_waiting_label;
}

/** Character Selection Screen *********
 * *************************/
void BustaGraphics::initCharSelScrn(gcn::ActionListener* al)
{
		gui_top = new gcn::Container();
		gui_top->setDimension(gcn::Rectangle(0, 0, SCREEN_RES_WIDTH, SCREEN_RES_WIDTH));
		gui_top->setOpaque(false);
		gui = new gcn::Gui();
		gui->setGraphics(gui_graphics);
		gui->setInput(gui_input);
		gui->setTop(gui_top);

		//Initialize widgets
		//Chef buttons
		char filename[MAX_FILE_LENGTH];
		char event_id[32];
		int x = CHAR_SEL_CHEFS_ALIGN_X;
		int y = CHAR_SEL_CHEFS_ALIGN_Y;
		for(int i=0; i<NUM_CHEFS; i++)
		{
			//Loads the first bmp of the healthy chef
			sprintf(filename, "%s/%s/Chef%d/Healthy/0.bmp", PATH_SPRITES, PATH_CHEF_SPRITES, i);
			chef_buttons[i] = new ChefButton(filename, CHEF_WIDTH, CHEF_HEIGHT);
			//Sets event_ID to Chef0, Chef1,..., Chef7
			sprintf(event_id, "Chef%d", i);
			chef_buttons[i]->setEventId(event_id);
			chef_buttons[i]->addActionListener(al);
			chef_buttons[i]->setBackgroundColor(CHAR_SEL_CHEF_BACKGROUND);
			gui_top->add(chef_buttons[i], x, y);
			if (i == 3){
				x = CHAR_SEL_CHEFS_ALIGN_X;
				y += CHEF_HEIGHT + CHAR_SEL_CHEFS_GAP;
			}
			else
				x += CHEF_WIDTH + CHAR_SEL_CHEFS_GAP;
		}
		//Cancel Button
		char_sel_cancel_button = new gcn::Button(CHAR_SEL_CANCEL_BUTTON_ID);
		char_sel_cancel_button->setEventId(CHAR_SEL_CANCEL_BUTTON_ID);
		char_sel_cancel_button->addActionListener(al);
		gui_top->add(char_sel_cancel_button, CHAR_SEL_CANCEL_BUTTON_X, CHAR_SEL_CANCEL_BUTTON_Y );
		
		//Select Button
		char_sel_select_button = new gcn::Button(CHAR_SEL_SELECT_BUTTON_ID);
		char_sel_select_button->setEventId(CHAR_SEL_SELECT_BUTTON_ID);
		char_sel_select_button->addActionListener(al);
		gui_top->add(char_sel_select_button, CHAR_SEL_SELECT_BUTTON_X, CHAR_SEL_SELECT_BUTTON_Y );

}

void BustaGraphics::killCharSelScrn()
{
	delete gui_top;
	delete gui;
	delete char_sel_cancel_button;
	delete char_sel_select_button;

	for(int i=0; i<NUM_CHEFS; i++)
		delete chef_buttons[i];
}
void BustaGraphics::drawCharSelScrn(int local_row, int local_col, int* others_row, int* others_col, char** player_names, int num_players)
{
		SDL_Rect rect;
		rect.x = 0; rect.y = 0;
		updateRect(&rect, mySprites->getGameBackground());


		myFont->write(myScreen, "Character Selection", CHAR_SEL_CHEFS_ALIGN_X, CHAR_SEL_CHEFS_ALIGN_Y - 60);

		//Draw Local player's cursor
		rect.x = CHAR_SEL_CHEFS_ALIGN_X + local_col*(CHEF_WIDTH+CHAR_SEL_CHEFS_GAP) - 7 ;
		rect.y = CHAR_SEL_CHEFS_ALIGN_Y + local_row*(CHEF_HEIGHT+CHAR_SEL_CHEFS_GAP) - 7;
		rect.w = CHEF_WIDTH + 14;
		rect.h = CHEF_HEIGHT + 14;
		SDL_FillRect(myScreen, &rect, CHAR_SEL_CURSOR_COLOR);

		//Draw network player's cursor
		for (int i=0; i<num_players; i++)
		{
				rect.x = CHAR_SEL_CHEFS_ALIGN_X + others_col[i]*(CHEF_WIDTH+CHAR_SEL_CHEFS_GAP) - 5 ;
				rect.y = CHAR_SEL_CHEFS_ALIGN_Y + others_row[i]*(CHEF_HEIGHT+CHAR_SEL_CHEFS_GAP) - 5;
				rect.w = CHEF_WIDTH + 10;
				rect.h = CHEF_HEIGHT + 10;
				SDL_FillRect(myScreen, &rect, CHAR_SEL_OTHER_CURSOR_COLOR);
				myFont->write(myScreen, player_names[i], rect.x, rect.y-30);
				
		}
		
		gui->draw();

		SDL_Flip(myScreen);
}
/** Key Binding screen *******
/*****************************/

void BustaGraphics::initKeyBindingScrn(gcn::ActionListener* al)
{
		gui_top = new gcn::Container();
		gui_top->setDimension(gcn::Rectangle(0, 0, SCREEN_RES_WIDTH, SCREEN_RES_WIDTH));
		gui_top->setOpaque(false);
		gui = new gcn::Gui();
		gui->setGraphics(gui_graphics);
		gui->setInput(gui_input);
		gui->setTop(gui_top);


				//Initialize widgets
		kb_ok_button = new gcn::Button(KB_OK_BUTTON_ID);
		kb_ok_button->setEventId(KB_OK_BUTTON_ID);
		kb_ok_button->addActionListener(al);
		kb_ok_button->setFocusable(false);

		kb_customer_submit_button = new gcn::Button(KB_CUSTOMER_SUBMIT_BUTTON_ID);
		kb_customer_submit_button->setEventId(KB_CUSTOMER_SUBMIT_BUTTON_ID);
		kb_customer_submit_button->addActionListener(al);
		kb_customer_submit_button->setFocusable(false);

		kb_opponent_submit_button = new gcn::Button(KB_OPPONENT_SUBMIT_BUTTON_ID);
		kb_opponent_submit_button ->setEventId(KB_OPPONENT_SUBMIT_BUTTON_ID);
		kb_opponent_submit_button ->addActionListener(al);
		kb_opponent_submit_button ->setFocusable(false);

		kb_self_submit_button = new gcn::Button(KB_SELF_SUBMIT_BUTTON_ID);
		kb_self_submit_button->setEventId(KB_SELF_SUBMIT_BUTTON_ID);
		kb_self_submit_button->addActionListener(al);
		kb_self_submit_button->setFocusable(false);

		kb_swap_button = new gcn::Button(KB_SWAP_BUTTON_ID);
		kb_swap_button->setEventId(KB_SWAP_BUTTON_ID);
		kb_swap_button->addActionListener(al);
		kb_swap_button->setFocusable(false);

		kb_rotate_button = new gcn::Button(KB_ROTATE_BUTTON_ID);
		kb_rotate_button->setEventId(KB_ROTATE_BUTTON_ID);
		kb_rotate_button->addActionListener(al);
		kb_rotate_button->setFocusable(false);

		kb_left_button = new gcn::Button(KB_LEFT_BUTTON_ID);
		kb_left_button->setEventId(KB_LEFT_BUTTON_ID);
		kb_left_button->addActionListener(al);
		kb_left_button->setFocusable(false);

		kb_right_button = new gcn::Button(KB_RIGHT_BUTTON_ID);
		kb_right_button->setEventId(KB_RIGHT_BUTTON_ID);
		kb_right_button->addActionListener(al);
		kb_right_button->setFocusable(false);

		kb_up_button = new gcn::Button(KB_UP_BUTTON_ID);
		kb_up_button->setEventId(KB_UP_BUTTON_ID);
		kb_up_button->addActionListener(al);
		kb_up_button->setFocusable(false);

		kb_down_button = new gcn::Button(KB_DOWN_BUTTON_ID);
		kb_down_button->setEventId(KB_DOWN_BUTTON_ID);
		kb_down_button->addActionListener(al);
		kb_down_button->setFocusable(false);

		gui_top->add(kb_customer_submit_button, KB_BUTTONS_TOP_X, KB_BUTTONS_TOP_Y);
		gui_top->add(kb_opponent_submit_button, KB_BUTTONS_TOP_X, KB_BUTTONS_TOP_Y + 30);
		gui_top->add(kb_self_submit_button, KB_BUTTONS_TOP_X, KB_BUTTONS_TOP_Y+60);
		gui_top->add(kb_swap_button, KB_BUTTONS_TOP_X, KB_BUTTONS_TOP_Y+90);
		gui_top->add(kb_rotate_button, KB_BUTTONS_TOP_X, KB_BUTTONS_TOP_Y+120);
		gui_top->add(kb_swap_button, KB_BUTTONS_TOP_X, KB_BUTTONS_TOP_Y+150);
		gui_top->add(kb_left_button, KB_BUTTONS_TOP_X, KB_BUTTONS_TOP_Y+180);
		gui_top->add(kb_right_button, KB_BUTTONS_TOP_X, KB_BUTTONS_TOP_Y+210);
		gui_top->add(kb_up_button, KB_BUTTONS_TOP_X, KB_BUTTONS_TOP_Y+240);
		gui_top->add(kb_down_button, KB_BUTTONS_TOP_X, KB_BUTTONS_TOP_Y+270);

		gui_top->add(kb_ok_button, 600, 500);
}

void BustaGraphics::drawKeyBindingScrn(bool waitingForKey)
{
		SDL_Rect rect;
		rect.x = 0; rect.y = 0;
		updateRect(&rect, mySprites->getGameBackground());

		rect.x = CS_BG_BOX_X; 
		rect.y = CS_BG_BOX_Y;
		rect.w = CS_BG_BOX_WIDTH;
		rect.h = CS_BG_BOX_HEIGHT;
		SDL_FillRect(myScreen, &rect, SP_BG_COLOR);

		if(waitingForKey)
		{
			myFont->write(myScreen, "Please press a key.", 200, 300);
		}

		gui->draw();
		SDL_Flip(myScreen);

}

void BustaGraphics::killKeyBindingScrn()
{
		delete gui_top;
		delete gui;
		delete kb_customer_submit_button;
		delete kb_opponent_submit_button;
		delete kb_self_submit_button;
		delete kb_swap_button;
		delete kb_rotate_button;
		delete kb_left_button;
		delete kb_right_button;
		delete kb_up_button;
		delete kb_down_button;
}


void BustaGraphics::initHighScores(gcn::ActionListener* al){
		gui_top = new gcn::Container();
		gui_top->setDimension(gcn::Rectangle(0, 0, SCREEN_RES_WIDTH, SCREEN_RES_HEIGHT));
		gui_top->setOpaque(false);

		gui = new gcn::Gui();
		gui->setGraphics(gui_graphics);
		gui->setInput(gui_input);
		gui->setTop(gui_top);

		hs_ok_button = new gcn::Button("OK");
		hs_ok_button->setEventId("OK");
		hs_ok_button->addActionListener(al);
		gui_top->add(hs_ok_button, HS_OKBUTTON_X, HS_OKBUTTON_Y );

}
void BustaGraphics::killHighScores(){
	delete gui_top;
	delete gui;
	delete hs_ok_button;
}

void BustaGraphics::drawHighScores(char** HSnames,int* HSscores )
{
	fillScreen(MS_BG_COLOR_BLUE);
	char buffer[10];
	myFont->writeCentered(myScreen, "High Scores", HS_TITLE_Y);
	for(int i=0;i<10;i++){
		myFont->write(myScreen,itoa(i+1,buffer,10),HS_NAMELEFT,HS_LISTTOP + HS_ENTRYHEIGHT * i);
		myFont->write(myScreen,HSnames[i],HS_NAMELEFT + 40,HS_LISTTOP + HS_ENTRYHEIGHT * i);
		myFont->write(myScreen,itoa(HSscores[i],buffer,10),HS_SCORELEFT,HS_LISTTOP + HS_ENTRYHEIGHT * i);
	}
	gui->draw();
	SDL_Flip(myScreen);
	
}

void BustaGraphics::initRankings(gcn::ActionListener* al)
{
	gui_top = new gcn::Container();
	gui_top->setDimension(gcn::Rectangle(0, 0, SCREEN_RES_WIDTH, SCREEN_RES_HEIGHT));
	gui_top->setOpaque(false);

	gui = new gcn::Gui();
	gui->setGraphics(gui_graphics);
	gui->setInput(gui_input);
	gui->setTop(gui_top);

	rs_playagain_button = new gcn::Button("Play Again");
	rs_playagain_button->setEventId("Play Again");
	rs_playagain_button->addActionListener(al);
	gui_top->add(rs_playagain_button, RS_PLAYAGAINBUTTON_X, RS_PLAYAGAINBUTTON_Y );

	rs_mainmenu_button = new gcn::Button("Main Menu");
	rs_mainmenu_button->setEventId("Main Menu");
	rs_mainmenu_button->addActionListener(al);
	gui_top->add(rs_mainmenu_button, RS_MAINMENUBUTTON_X, RS_MAINMENUBUTTON_Y );
}
void BustaGraphics::killRankings()
{
	delete gui_top;
	delete gui;
	delete rs_playagain_button;
	delete rs_mainmenu_button;
}
void BustaGraphics::drawRankings(char** names, int numPlayers, BustaModelCharacter* portraits, int* scores)
{
	fillScreen(SP_BG_COLOR);
	char buffer[10];
	SDL_Rect rect;
	myFont->writeCentered(myScreen, "Rankings", RS_TITLE_Y);
	for(int i=0;i<numPlayers;i++){
		myFont->write(myScreen,itoa(i+1,buffer,10),RS_NAMELEFT,RS_LISTTOP + RS_ENTRYHEIGHT * i);
		myFont->write(myScreen,names[i],RS_NAMELEFT + 25,RS_LISTTOP + RS_ENTRYHEIGHT * i);
		myFont->write(myScreen,itoa(scores[i],buffer,10),RS_NAMELEFT + 325,RS_LISTTOP + RS_ENTRYHEIGHT * i);
		rect.x = RS_NAMELEFT - 50;
		rect.y = 4 + RS_LISTTOP + RS_ENTRYHEIGHT * i;
		updateRect(&rect, mySprites->getChefPortraits(portraits[i]));
	}
	gui->draw();

	SDL_Flip(myScreen);
}

void BustaGraphics::drawOptions()
{
		printf("Called drawOptions\n");
}

//button stuff
void BustaGraphics::initButtonLocs()
{
	bMainScrn[MS_START_SP].box.x = MS_START_SP_POS_X;
	bMainScrn[MS_START_SP].box.y = MS_START_SP_POS_Y;
	bMainScrn[MS_START_SP].box.w = MS_START_SP_W;
	bMainScrn[MS_START_SP].box.h = MS_START_SP_H;
	bMainScrn[MS_START_SP].nextLink = &bMainScrn[MS_START_MP];
	bMainScrn[MS_START_SP].prevLink = &bMainScrn[MS_QUIT];

	bMainScrn[MS_START_MP].box.x = MS_START_MP_POS_X;
	bMainScrn[MS_START_MP].box.y = MS_START_MP_POS_Y;
	bMainScrn[MS_START_MP].box.w = MS_START_MP_W;
	bMainScrn[MS_START_MP].box.h = MS_START_MP_H;
	bMainScrn[MS_START_MP].nextLink = &bMainScrn[MS_HIGH_SCORE];
	bMainScrn[MS_START_MP].prevLink = &bMainScrn[MS_START_SP];

	bMainScrn[MS_HIGH_SCORE].box.x = MS_HIGH_SCORE_POS_X;
	bMainScrn[MS_HIGH_SCORE].box.y = MS_HIGH_SCORE_POS_Y;
	bMainScrn[MS_HIGH_SCORE].box.w = MS_HIGH_SCORE_W;
	bMainScrn[MS_HIGH_SCORE].box.h = MS_HIGH_SCORE_H;
	bMainScrn[MS_HIGH_SCORE].nextLink = &bMainScrn[MS_OPTIONS];
	bMainScrn[MS_HIGH_SCORE].prevLink = &bMainScrn[MS_START_MP];

	bMainScrn[MS_OPTIONS].box.x = MS_OPTIONS_POS_X;
	bMainScrn[MS_OPTIONS].box.y = MS_OPTIONS_POS_Y;
	bMainScrn[MS_OPTIONS].box.w = MS_OPTIONS_W;
	bMainScrn[MS_OPTIONS].box.h = MS_OPTIONS_H;
	bMainScrn[MS_OPTIONS].nextLink = &bMainScrn[MS_QUIT];
	bMainScrn[MS_OPTIONS].prevLink = &bMainScrn[MS_HIGH_SCORE];

	bMainScrn[MS_QUIT].box.x = MS_QUIT_POS_X;
	bMainScrn[MS_QUIT].box.y = MS_QUIT_POS_Y;
	bMainScrn[MS_QUIT].box.w = MS_QUIT_W;
	bMainScrn[MS_QUIT].box.h = MS_QUIT_H;
	bMainScrn[MS_QUIT].nextLink = &bMainScrn[MS_START_SP];
	bMainScrn[MS_QUIT].prevLink = &bMainScrn[MS_OPTIONS];
	
}


			
		
