/* 
 * BustaGraphics.h
 * Busta' Sandwich Graphics Engine
 * Authors: Julian Chan, Thomas Patecky
 *
 * Created: July 13, 2005
 * Last Modified July 15, 2005
 */

#ifndef BUSTAGRAPHICS_H
#define BUSTAGRAPHICS_H

#include <stdio.h>
#include <stdlib.h>
#include "../../SDL/include/SDL.h"
//#include "../../GUIChan/guichan.hpp"
//#include "../../GUIChan/guichan/sdl.hpp"
#include "./GUIChan/include/guichan.hpp"
#include "./GUIChan/include/guichan/sdl.hpp"
#include "../models/BustaModelGameState.h"
#include "../models/BustaModelCompleteSandwich.h"
#include "../models/BustaModelOpponent.h"
#include "../models/BustaModelMPGameState.h"
#include "BustaGraphicsSprites.h"
#include "BustaGraphicsLayout.h"
#include "Font.h"

/*
//Mock-up MPGameState to test drawMPGame()
class BustaModelMPGameState
{
		public:
		BustaModelGameState *game1, *game2;
		BustaModelOpponent *opponent1, *opponent2;

		BustaModelMPGameState();
		BustaModelMPGameState(BustaModelGameState* gs1, BustaModelOpponent* op1, BustaModelGameState* gs2, BustaModelOpponent* op2)
		{
				game1 = gs1;
				game2 = gs2;
				opponent1 = op1;
				opponent2 = op2;
		}

		BustaModelGameState* getGameState() { return game1;}

		BustaModelOpponent* getOpponent(int n)
		{
				if (n==0) return opponent1;
				return opponent2;
		}
};
*/

struct Button {
	SDL_Rect box;
	Button* nextLink;
	Button* prevLink;
};

class ChefButton : public gcn::Button
{
	private:
		gcn::Image* image;
		int width, height;
	public:
		ChefButton(char* image_filename, int w, int h)
		{
			Button();
			try
			{
				image = new gcn::Image(image_filename);
			}
			catch (gcn::Exception e)
			{
				printf("Caught GUIChan exception: %s\n", e.getMessage().c_str());
			}
            		
			setWidth(w);
			setHeight(h);
		}

		~ChefButton() { delete image; }

		void draw(gcn::Graphics* graphics)
		{
			graphics->drawImage(image,0,0);
			if (isPressed())
				graphics->drawRectangle(gcn::Rectangle(2,2,getWidth()-4,getHeight()-4));
				
		}
};


class OpponentButton : public gcn::Button
{
	private:
		gcn::Image* image;
		int width, height;
		BustaModelOpponent* opponent;
		char* name;
		bool opponentIsSet;
	public:
		OpponentButton(char* op_name, gcn::Image* img, int w, int h)
		{
			Button();
			setWidth(w);
			setHeight(h);
			image = img;
			name = op_name;
			setFocusable(false);
			opponentIsSet = false;
		}

		void setOpponent(BustaModelOpponent *op)
		{
			opponentIsSet = true;
			opponent = op;
		}

		~OpponentButton() { delete image; }

		void draw(gcn::Graphics* graphics)
		{
			int x,y,w,h;
			//Draw white bg, name and portrait
			graphics->setColor(gcn::Color(255,255,255));
			graphics->fillRectangle(gcn::Rectangle(0,0,getWidth(),getHeight()));
			graphics->drawText(name, 5, 0, LEFT);
			graphics->drawImage(image,5,15);
			
			//Draw the opponent's status
			if (opponentIsSet)
			{
				graphics->drawText("H", MP_OPPONENT_PORTRAIT_SIZE + 10, 15, getAlignment());
				
				//Health bar bg
				x = MP_OPPONENT_PORTRAIT_SIZE+15;
				y = 18;
				w = MP_OPPONENT_BUTTON_WIDTH - MP_OPPONENT_PORTRAIT_SIZE - 15 - 5;
				h = 10;
				graphics->setColor(gcn::Color(HEALTH_BAR_BG_COLOR));
				graphics->fillRectangle(gcn::Rectangle(x, y, w, h));
				//"blood" remaining
				BustaModelChef* chef = new BustaModelChef(); //so that I can call getMaxHealth
				w = w * opponent->getHealth() / (chef->getMaxHealth());
				graphics->setColor(gcn::Color(HEALTH_BAR_FG_COLOR));
				graphics->fillRectangle(gcn::Rectangle(x, y, w, h));
				delete chef;
				
				graphics->drawText("$", MP_OPPONENT_PORTRAIT_SIZE + 10, 30, getAlignment());

				//Money bar bg
				y = 30;
				w = MP_OPPONENT_BUTTON_WIDTH - MP_OPPONENT_PORTRAIT_SIZE - 15 - 5;
				graphics->setColor(gcn::Color(HEALTH_BAR_BG_COLOR));
				graphics->fillRectangle(gcn::Rectangle(x, y, w, h));
				
				//money remaining
				BustaModelTipJar* tipjar = new BustaModelTipJar();
				w = w * opponent->getTips() / (tipjar->getSize());
				graphics->setColor(gcn::Color(MP_OPPONENT_MONEY_BAR_COLOR));
				graphics->fillRectangle(gcn::Rectangle(x, y, w, h));
				delete tipjar;
			}

			if (isPressed())
			{	
				graphics->setColor(gcn::Color(0,0,0));
				graphics->drawRectangle(gcn::Rectangle(2,2,getWidth()-4,getHeight()-4));
			}
				
		}
};

// Connection screen GUI stuff
// note: edit buttons position and such in BustaGraphicsLayout.h
// The engine must specify what state the screen is in.
enum ConnectScrnStates {CS_NOT_CONNECTED, CS_CONNECTING, CS_CONNECTED,CS_READY,CS_CANCEL};

struct ConnectScrnPlayerList {
		char* name;
		int id;
		char* IP_address;
		bool isReady;
		ConnectScrnPlayerList* next;
};


/**
 * @class BustaGraphics
 * @brief contains methods for displaying graphics
 *
 */
class BustaGraphics
{
	private:
		//The screen to draw things on
		SDL_Surface* myScreen;
		//All the sprites
		BustaGraphicsSprites* mySprites;
		//BMP font
		SFont* myFont;
		SFont* myChalkFont;

		//Special Sandwiches
		bool special_menu_is_set;
		int num_special_sws_sp;
		int num_special_sws;

		FoodType** special_sw;
		char** special_sw_names;

		//How many frames have been rendered in total
		int overall_anim_frame;

		int chef_anim_frame;
		int customer_anim_frame;

		//Universal GUI Stuff
		gcn::ImageLoader* gui_image_loader;
		gcn::SDLGraphics* gui_graphics;
		gcn::Container* gui_top;
		gcn::Gui* gui;
		gcn::ImageFont* gui_font;
		gcn::SDLInput* gui_input;

		//Connection Screen GUI Stuff
		gcn::Label* connect_scrn_server_ip_label;
		gcn::TextField* connect_scrn_server_ip_field;
		gcn::Label* connect_scrn_name_label;
		gcn::Label* connect_scrn_waiting_label;
		gcn::TextField* connect_scrn_name_field;
		gcn::Button* connect_scrn_connect_button;
		gcn::Button* connect_scrn_cancel_button;
		gcn::Button* connect_scrn_ready_button;

		//SP Game GUI stuff
		gcn::Button* submit_self_button;
		gcn::Button* submit_customer_button;
		gcn::Button* end_game_button;
		gcn::TextBox* history_text_box;

		//MP Game GUI stuff (SP GUI is reused)
		OpponentButton* opponent_buttons[MAX_OPPONENTS];
		gcn::Button* submit_opponent_button;
		gcn::Button* show_special_button;

		//Character Selection Screen GUI stuff
		ChefButton* chef_buttons[NUM_CHEFS];
		gcn::Button* char_sel_select_button;
		gcn::Button* char_sel_cancel_button;

		//Key Binding screen gui stuff
		gcn::Button* kb_customer_submit_button;
		gcn::Button* kb_opponent_submit_button;
		gcn::Button* kb_self_submit_button;
		gcn::Button* kb_swap_button;
		gcn::Button* kb_rotate_button;
		gcn::Button* kb_left_button;
		gcn::Button* kb_right_button;
		gcn::Button* kb_up_button;
		gcn::Button* kb_down_button;
		gcn::Button* kb_ok_button;

		//High Score Screen GUI stuff
		gcn::Button* hs_ok_button;

		//Rankings screen GUI stuff
		gcn::Button* rs_playagain_button;
		gcn::Button* rs_mainmenu_button;

		//timing
		int chefLastTime;
		int chefCurrTime;

		//sandwich crusher stuff
		int crusherState; //0:CrusherOpenNothingInside, 1:CrusherOpenFoodInside, 2:CrusherClosed
		int crusherLastTime;
		int crusherCurrTime;
		BustaModelCompleteSandwich* inCrusher;


		//Private drawing functions

		/**
		 * @brief Draws board
		 * @param x the x-pos of the upper-left corner of upper-leftmost piece
		 * @param y y-pos
		 * @param gs A pointer to the BustaModelGameState.
		 */
		void drawBoard(int x, int y, BustaModelGameState* gs);

		/**
		 * @brief Draws a food tile
		 * @param food_piece A pointer to the BustaModelFoodPiece.
		 * @param x x-pos of the upper-left corner of the piece.
		 * @param y y-pos.
		 */
		void drawFoodPiece(BustaModelFoodPiece* food_piece, int x, int y);

		/**
		 * @brief Draws the special sandwich menu at x,y
		 */
		void drawSpecialSandwichMenu(int x, int y, bool multiplayer);
		
		/**
		 * @brief Draws a small food tile
		 * @param food_piece A pointer to the BustaModelFoodPiece.
		 * @param x x-pos of the upper-left corner of the piece.
		 * @param y y-pos.
		 */
		void drawSmallFoodPiece(BustaModelFoodPiece* food_piece, int x, int y);

		/**
		* @brief Draws the tip jar
		* @param tj A pointer to the Tip Jar Model.
		*/
		void drawTipJar(int x, int y, BustaModelTipJar* tj);

		/**
		* @brief Draws the score. Pre-condition: score must not be > 10^16
		* @param x x-pos of where the score will be drawn
		* @param y y-pos
		* @param score The Score.
		*/
		void drawScore(int x, int y, int score);
		
	
		/**
		* @brief Draws the cursor 
		* @param board_x x-pos of the upper-left corner of the upper-leftmost piece of the board
		* @param board_y ditto, but the y-pos 
		* @param row row number of the cursor on the board.
		* @param col ditto, but the col number.
		* @param swap_alignment alignment of the swap cursor {LEFT, RIGHT, UP, DOWN}
		*/
		void drawCursor(int board_x, int board_y, int row, int col, int swap_alignment);
	
		/**
		* @brief clears the screen with the specified color
		* @param 
		*/
		void fillScreen(Uint32 color);

		/**
		* @brief Draws the chef
		* @param x x-pos of the top-left corner of the chef
		* @param y ditto, but the y-pos.
		* @param c A pointer to the Chef Model.
		*/

		void drawChef(int x, int y, BustaModelChef* c);	
		

		/**
		* @brief Callback function updates the animation frame on timer event
		* @param interval The time interval passed.
		* @param param Junk.
		* @return interval.
		*/
		/*Uint32 updateChefFrame(Uint32 interval, void *param);*/

		/**
		* @brief Draws the chef
		* @param x x-pos of the top-left corner of the bar
		* @param y y-pos
		* @param health current health of the chef.
		* @param max_health the max possible health of the chef
		*/
		void drawHealthBar(int x, int y, int health, int max_health);
		
		/**
		* @brief Draws the customer
		* @param x x-pos of the top-left corner of the first piece
		* @param y y-pos
		* @param customer a pointer to the current customer
		*/
		void drawCustomer(int x, int y, BustaModelCustomer* customer);

		/**
		* @brief Draws the customer's order with overlapping food pieces
		* @param x x-pos of the top-left corner of the first piece
		* @param y y-pos
		* @param order_array customer's order
		*/
		void drawOrder(int x, int y, BustaModelFoodPiece** order_array);
		
		/**
		* @brief Draws the sandwich crusher
		*/
		void drawCrusher();

		/**
		 * @brief update a rectangle on the screen
		 * @param rect a rectangle on the screen
		 * @param rectSurface image to update the rectangle with
		 */
		void updateRect(SDL_Rect* rect, SDL_Surface* rectSurface);

				
		/**
		 * @brief intialize the button parameters
		 */
		void initButtonLocs();
	
	public:
		BustaGraphics();
		BustaGraphics(FoodType** special_sandwiches, char** sandwich_names, int num_sandwiches, int num_sandwiches_sp);
		BustaGraphics(int a);
		BustaGraphics(SDL_Surface* screen);

		~BustaGraphics()
		{
			delete gui_image_loader;
			delete gui_graphics;
			delete gui_font;
			delete gui_input;
			SDL_FreeSurface(myScreen);
			delete mySprites;
			delete myFont;

			//Set special sandwiches
			for(int i=0; i<num_special_sws; i++)
			{
					delete special_sw[i];

					delete special_sw_names[i];
			}

		}
		
		/**
		* @brief Alerts the GUI when a sandwich is submitted
		* @param sandwich The sandiwch submitted.
		*/
		void submissionAlert(BustaModelCompleteSandwich* sandwich);

		/**
		 * @brief Set the special sandwiches
		 */
		void setSpecialSandwiches(FoodType** special_sandwiches, char** sandwich_names, int num_sandwiches, int num_spSandwiches);
		
		/**
		 * @brief Pass on any SDL_Event to see if it has anything to do with the GUI
		 */
		void handleGUIEvent(SDL_Event* event);
		
		
		/*******************************************
		 * Single Player Game Screen 
		 *******************************************/	
		
		void initSPGameScrn(gcn::ActionListener* al);
		
		
		/**
		* @brief Draws the singleplayer game. Calls SDL_Flip()
		* @param gs A pointer to the Game State.
		*/
		void drawSPGame(BustaModelGameState* gs);
		
		/**
		* @brief Draws the game state of the local player without calling SDL_Flip()
		* @param gs A pointer to the Game State.
		*/
		void drawLocalPlayerGame(BustaModelGameState* gs);

		void newdrawSPGame(BustaModelGameState* gs);

		/**
		 * @brief Add a new line to the history, scrolling the old text up.Will also wrap the text for you 
		 * @param string The new line.
		 */
		void addHistory(const char* string);

		/**
		 * @brief Free all the gui stuff
		 */
		void killSPGameScrn();

		
		/**
		* @brief Draws the main screen
		*/
		void drawMainScreen(MainScrnButton selected);
	
		
		/*******************************************
		 * Connection Screen
		 *******************************************/	
		/**
		 * @brief Initialize the connection screen
		 * @param al A pointer to an ActionListener so the GUI knows who to notify when something happens
		 */
	
		void initConnectScrn(gcn::ActionListener* al);


		/**
		* @brief Draws the connect screen
		*/
		void drawConnectScrn(ConnectScrnStates state, ConnectScrnPlayerList* list);
		
		/**
		* @brief Returns the text in the text field. Returns an empty if the field is uninitialized
		*/
		const char* getConnectScrnTextField();

		/**
		* @brief Returns the server IP address typed in by the player
		*/
		const char* getServerIPAddress();
		
		/**
		* @brief Returns the name typed in by the player
		*/
		const char* getPlayerName();
		/**
		 * @brief Kill the connect screen. Free up some precious memory
		 */
		void killConnectScrn();

		/***** End of Connection Screen *****/

		/*******************************************
		 * Character Selection Screen
		 *******************************************/	
		/**
		 * @brief Initialize the character selection screen
		 * @param al A pointer to an ActionListener so the GUI knows who to notify when something happens
		 */
	
		void initCharSelScrn(gcn::ActionListener* al);


		/**
		* @brief Draws the character selection screen
		* @param local_row row number of the selected chef of the local player
		* @param local_col ditto, but the col number
		* @param others_row an array of the row numbers all the other network players current selection
		* @param others_col ditto, but the col numbers
		* @param player_names an array of player names
		* @param num_players the number of network players
		*/
		void drawCharSelScrn(int local_row, int local_col, int* others_row, int* others_col, char** player_names, int num_players);
		

		/**
		 * @brief Kill the connect screen. Free up some precious memory
		 */
		void killCharSelScrn();

		
		/*******************************************
		 * MP Game Screen
		 * *****************************************/

		/**
		 * @brief Initialize the Multiplayer game screen
		 */

		void initMPGameScrn(gcn::ActionListener* al, BustaModelCharacter* chefID, char** opponent_names, int num_opponents);

		//void initMPGameScrn(gcn::ActionListener* al);

		/**
		 * @brief set the opponents
		 */
		void setOpponents(BustaModelMPGameState* mp_gs, int num_opponents);
		
		/**
		* @brief Draws a window containg a multiplayer game
		* @param gs A pointer to the Game State.
		*/
		void drawMPGame(BustaModelGameState* gs, BustaModelMPGameState* mp_gs);
		void drawMPGame(BustaModelGameState* gs, BustaModelMPGameState* net_player_gs, bool drawSpecialMenu, int num_opponents);
		//void drawMPGame(BustaModelGameState gs);

		/**
		 * @brief Free up the memory used by the multiplayer game screen
		 */
		void killMPGameScrn();
		
		/**
		* @brief Draws the opponent's game board
		* @param gs A pointer to the Game State.
		*/
		void drawOpponentBoard(BustaModelGameState gs);

		//Other screens


		/*******************************************
		 * Chef Selection Screen
		 * *****************************************/

		/**
		* @brief Draws the chef selection screen
		*/
		void drawChefSelelection();


		void initKeyBindingScrn(gcn::ActionListener* al);
		void drawKeyBindingScrn(bool waitingForKey);
		void killKeyBindingScrn();

		/**
		* @brief Initializes the high scores screen
		* @param al A pointer to an ActionListener so the GUI knows who to notify when something happens
		*/
		void initHighScores(gcn::ActionListener* al);
		/**
		* @brief Kills the high scores screen, call on leaving High Score screen.
		*/
		void killHighScores();
		/**
		* @brief Draws the high scores screen, first call initHighScores
		* @param HSnames A pointer to an array containing name strings
		* @param HSscores A pointer to an int array containing scores
		*/
		void drawHighScores(char** HSnames,int* HSscores );

		/**
		* @brief Draws the rankings screen
		* @param al A pointer to an ActionListener so the GUI knows who to notify when something happens
		*/
		void BustaGraphics::initRankings(gcn::ActionListener* al);

		/**
		* @brief Kills the rankings screen
		*/
		void BustaGraphics::killRankings();

		/**
		* @brief Draws the rankings screen
		* @param names A pointer to an array containing the player names, in order of rank (1st place, 2nd place, etc.)
		* @param numPlayers the number of players
		*/
		void BustaGraphics::drawRankings(char** names, int numPlayers, BustaModelCharacter* portraits, int* scores);

		/**
		* @brief Draws the options screen
		*/
		void drawOptions();


		/**
		* @brief Data Structure containing the button locations and links
		*/
		Button bMainScrn[MS_NUM_BUTTONS];

};


#endif
