#include "BustaGraphicsSprites.h"


//Load all the sprites
BustaGraphicsSprites::BustaGraphicsSprites()
{
		LoadGameBackground();
		LoadMainScreenLogo();
		LoadFoodPieces();
		LoadSmallFoodPieces();
		LoadFonts();
		LoadChefSprites();
		LoadCursorSprites();
		LoadTipJarSprites();
		LoadCustomerSprites();
		LoadConnectScrnStatusLights();
		LoadChalkboard();
		LoadChefPortraits();
		LoadCrusher();

}

BustaGraphicsSprites::~BustaGraphicsSprites()
{
	int i,j,k;
	
	SDL_FreeSurface(game_background);
	SDL_FreeSurface(chalkboard);
	SDL_FreeSurface(cursor_sprites);
	SDL_FreeSurface(swap_cursor_sprites);
	SDL_FreeSurface(main_screen_logo);

	for(i=0; i<NUM_FONTS; i++)
			SDL_FreeSurface(fonts[i]);

	for(i=0; i<NUMFOODPIECES; i++)
		for(j=0; j<FOOD_NUM_FRESH_STATES; j++)
			SDL_FreeSurface(food_sprites[i][j]);
	
	for(i=0; i<NUMFOODPIECES; i++)
		SDL_FreeSurface(food_sprites_small[i]);

	for(i=0; i<NUM_CHEFS; i++)
		for(j=0; j<CHEF_NUM_STATES; j++)
			for(k=0; k<ANIM_CHEF_NUM_FRAMES; k++)
				SDL_FreeSurface(chef_sprites[i][j][k]);

	for(i=0; i<TIP_JAR_STAGES; i++)
		SDL_FreeSurface(tip_jar_sprites[i]);

	for(i=0; i<NUM_CUSTOMERS; i++)
		for(j=0; j<CUSTOMER_NUM_STATES; j++)
				SDL_FreeSurface(customer_sprites[i][j]);
	
	for(i=0; i<CS_STATUS_LIGHT_NUM_COLORS; i++)
		SDL_FreeSurface(connect_scrn_status_lights[i]);
}

void BustaGraphicsSprites::ApplyTransparency(SDL_Surface* surface)
{
		//Set the transparent key
		SDL_SetColorKey(surface, SDL_SRCCOLORKEY, SDL_MapRGB(surface->format, TRANSPARENT_KEY_R, TRANSPARENT_KEY_G, TRANSPARENT_KEY_B));
}

bool BustaGraphicsSprites::LoadGameBackground()
{
		char filename[MAX_FILE_LENGTH];
		
		//Load the main cursor
		sprintf(filename, "%s/%s", PATH_SPRITES, BMP_GAME_BACKGROUND);
		if ( (game_background = SDL_LoadBMP(filename)) == NULL)
		{
				printf("Game Background Load Failed. Filename: %s\n", filename);
				return false;
		}
		return true;
}

bool BustaGraphicsSprites::LoadChalkboard()
{
		char filename[MAX_FILE_LENGTH];
		
		//Load the main cursor
		sprintf(filename, "%s/%s", PATH_SPRITES, BMP_CHALKBOARD);
		if ( (chalkboard = SDL_LoadBMP(filename)) == NULL)
		{
				printf("Chalkboard Load Failed. Filename: %s\n", filename);
				return false;
		}
		return true;
}

bool BustaGraphicsSprites::LoadConnectScrnStatusLights()
{
		char filename[MAX_FILE_LENGTH];

		//Load the green light
		sprintf(filename, "%s/%s/%s", PATH_SPRITES, PATH_CONNECT_SCRN, BMP_GREEN_LIGHT);
		if ( (connect_scrn_status_lights[GREEN_LIGHT] = SDL_LoadBMP(filename)) == NULL )
		{
				printf("Status Lights Sprite Load Failed on  %s!\n", filename);
				return false;
		}
		ApplyTransparency(connect_scrn_status_lights[GREEN_LIGHT]);

		//Load the red light
		sprintf(filename, "%s/%s/%s", PATH_SPRITES, PATH_CONNECT_SCRN, BMP_RED_LIGHT);
		if ( (connect_scrn_status_lights[RED_LIGHT] = SDL_LoadBMP(filename)) == NULL )
		{
				printf("Status Lights Sprite Load Failed on  %s!\n", filename);
				return false;
		}
		ApplyTransparency(connect_scrn_status_lights[RED_LIGHT]);

		return true;
}
		
//Only loads Chef0 for now
bool BustaGraphicsSprites::LoadChefSprites()
{
		char filename[MAX_FILE_LENGTH];	

		char* chef_states[CHEF_NUM_STATES] = {
				"Healthy",
				"Bloated",
				"Dying"};
	
		//Load the Idle sprites
		for (int chef_ID=0; chef_ID<NUM_CHEFS; chef_ID++)
		{
			for (int state=0; state<CHEF_NUM_STATES; state++)
			{
	
				for (int frame=0; frame<ANIM_CHEF_NUM_FRAMES; frame++)
				{
						sprintf(filename, "%s/%s/Chef%d/%s/%d.bmp", PATH_SPRITES, PATH_CHEF_SPRITES, chef_ID, chef_states[state] , frame);
						if ( (chef_sprites[chef_ID][state][frame] = SDL_LoadBMP(filename)) == NULL)
						{
								printf("Chef Sprites Load Failed on %s!\n", filename);
								return false;
						}
	
						ApplyTransparency(chef_sprites[chef_ID][state][frame]);
				}
			}
		}
		return true;
}

//Loads the 4 stages of the tip jar
bool BustaGraphicsSprites::LoadTipJarSprites()
{
		char filename[MAX_FILE_LENGTH];
		
		for (int i=0; i<TIP_JAR_STAGES; i++)
		{
			sprintf(filename, "%s/%s/%d-4Full.bmp", PATH_SPRITES, PATH_TIP_JAR, i);
			if ( (tip_jar_sprites[i] = SDL_LoadBMP(filename)) == NULL)
			{
					printf("Tip jar sprites Load Failed. Filename: %s\n", filename);
					return false;
			}
			ApplyTransparency(tip_jar_sprites[i]);
		}
		return true;
}

bool BustaGraphicsSprites::LoadMainScreenLogo()
{
		char filename[MAX_FILE_LENGTH];
		
		//Load the main cursor
		sprintf(filename, "%s/%s/%s", PATH_SPRITES, PATH_MAIN_SCREEN, BMP_MAIN_SCREEN_LOGO);
		if ( (main_screen_logo = SDL_LoadBMP(filename)) == NULL)
		{
				printf("Main Screen Logo Load Failed. Filename: %s\n", filename);
				return false;
		}
		ApplyTransparency(main_screen_logo);
		return true;
}

//Only loads Patron0 for now
bool BustaGraphicsSprites::LoadCustomerSprites()
{
		char filename[MAX_FILE_LENGTH];	

		char* customer_states[CUSTOMER_NUM_STATES] = {
				"Content",
				"Pissed"};
	
		//Load the Idle sprites
		for (int customer_ID=0; customer_ID<NUM_CUSTOMERS; customer_ID++)
		{
			for (int state=0; state<CUSTOMER_NUM_STATES; state++)
			{
						sprintf(filename, "%s/%s/Patron%d/%s/0.bmp", PATH_SPRITES, PATH_CUSTOMER_SPRITES, customer_ID, customer_states[state]);
						if ( (customer_sprites[customer_ID][state] = SDL_LoadBMP(filename)) == NULL)
						{
								printf("Customer Sprites Load Failed on %s!\n", filename);
								return false;
						}
						ApplyTransparency(customer_sprites[customer_ID][state]);
			}
		}
		return true;
}

bool BustaGraphicsSprites::LoadCursorSprites()
{
		char filename[MAX_FILE_LENGTH];
		
		//Load the main cursor
		sprintf(filename, "%s/%s/0.bmp", PATH_SPRITES, PATH_CURSOR);
		if ( (cursor_sprites = SDL_LoadBMP(filename)) == NULL)
		{
				printf("Cursor Load Failed. Filename: %s\n", filename);
				return false;
		}
		ApplyTransparency(cursor_sprites);
		
		//Load the swap cursor	
		sprintf(filename, "%s/%s/0.bmp", PATH_SPRITES, PATH_SWAP_CURSOR);
		if ( (swap_cursor_sprites = SDL_LoadBMP(filename)) == NULL)
		{
				printf("Swap Cursor Load Failed. Filename: %s\n", filename);
				return false;
		}
		ApplyTransparency(swap_cursor_sprites);

		return true;
}

//Load static FoodPiece sprites that resides in PATH_SPRITES/PATH_FOOD_PIECE_SPRITES/<FOOD>/
bool BustaGraphicsSprites::LoadFoodPieces()
{
		char filename[MAX_FILE_LENGTH];
		//The file list of food sprites. Follows the order of the enum FoodType
		char* food_sprites_dirs[NUMFOODPIECES] = {
					"Tomato", 
					"Onion", 
					"Lettuce", 
					"Cheese",
					"Salami",
					"Chicken",
					"Turkey",
					"Ham",
					"Tofu",
					"Mayo",
					"Mustard",
					"Pickles"};

		//The different freshness states have different file names. 
		//Follows the order of the enum FoodFreshness
		char* fresh_state_files[FOOD_NUM_FRESH_STATES] = {
					"fresh.bmp",
					"stale.bmp",
					"rotten.bmp"};
	
		for (int piece=0; piece<NUMFOODPIECES; piece++)
		{
				for (int freshness=0; freshness<FOOD_NUM_FRESH_STATES; freshness++)
				{
					sprintf(filename, "%s/%s/%s/%s", PATH_SPRITES, PATH_FOOD_PIECE_SPRITES, food_sprites_dirs[piece], fresh_state_files[freshness]);
					if ( (food_sprites[piece][freshness] = SDL_LoadBMP(filename)) == NULL)
					{
							printf("Sprites Load Failed on %s!\n", filename);
							return false;
					}
				
					ApplyTransparency(food_sprites[piece][freshness]);
				}
		}
		

		return true;
}

bool BustaGraphicsSprites::LoadSmallFoodPieces()
{
		char* small_food_sprites_files[NUMFOODPIECES] = {
					"tomato.bmp",
					"onion.bmp", 
					"lettuce.bmp", 
					"cheese.bmp", 
					"salami.bmp",
					"chicken.bmp",
					"turkey.bmp",
					"ham.bmp",
					"tofu.bmp",
					"mayo.bmp",
					"mustard.bmp",
					"pickle.bmp"};
	
		char filename[MAX_FILE_LENGTH];	
		for (int piece=0; piece<NUMFOODPIECES; piece++)
		{
				sprintf(filename, "%s/%s/%s", PATH_SPRITES, PATH_SMALL_FOOD_PIECE_SPRITES, small_food_sprites_files[piece] );
				if ( (food_sprites_small[piece] = SDL_LoadBMP(filename)) == NULL)
				{
						printf("Small Food Piece  Sprite Load Failed on %s!\n", filename);
						return false;
				}
				
				ApplyTransparency(food_sprites_small[piece]);
		}

		return true;
}

bool BustaGraphicsSprites::LoadChefPortraits(){
		char filename[MAX_FILE_LENGTH];
		for(int i=0; i<NUM_CHEFS; i++)
		{
				sprintf(filename, "%s/Chefs/Chef%d/small.bmp", PATH_SPRITES, i);
				if ( (chef_portraits[i] = SDL_LoadBMP(filename)) == NULL)
				{
						printf("Chef Portrait Sprite Load Failed on %s!\n", filename);
						return false;
				}
				
				ApplyTransparency(chef_portraits[i]);
		}
		return true;
}

bool BustaGraphicsSprites::LoadCrusher(){
	char filename[MAX_FILE_LENGTH];
	for(int i=0; i<2; i++)
		{
				sprintf(filename, "%s/SandwichCrusher/%d.bmp", PATH_SPRITES, i);
				if ( (crusher_sprites[i] = SDL_LoadBMP(filename)) == NULL)
				{
						printf("Crusher Sprite Load Failed on %s!\n", filename);
						return false;
				}
				
				ApplyTransparency(crusher_sprites[i]);
		}
		return true;
}


//Load the BMP font that resides in PATH_SPRITES
bool BustaGraphicsSprites::LoadFonts()
{
		char filename[MAX_FILE_LENGTH];
		sprintf(filename, "%s/%s", PATH_SPRITES, STD_FONT);
		if ( (fonts[STD] = SDL_LoadBMP(filename)) == NULL)
		{
				printf("Font Load Failed. Filename: %s\n", filename);
				return false;
		}
		ApplyTransparency(fonts[STD]);

		sprintf(filename, "%s/%s", PATH_SPRITES, CHALK_FONT);
		if ( (fonts[CHALK] = SDL_LoadBMP(filename)) == NULL)
		{
				printf("Font Load Failed. Filename: %s\n", filename);
				return false;
		}
		ApplyTransparency(fonts[CHALK]);

		return true;
}

SDL_Surface* BustaGraphicsSprites::getConnectScrnStatusLight(CS_STATUS_LIGHT light_color)
{
		return connect_scrn_status_lights[light_color];
}

//Returns the proper sprite for the chef
//**Only returns sprites for Chef0 for now!
SDL_Surface* BustaGraphicsSprites::getChefSprites(BustaModelCharacter chef, ANIM_CHEF_TYPES state, int frame)
{
		return chef_sprites[(int)chef][state][frame];
}

//Returns the proper sprite for the customer
//**Only returns sprites for Patron0 for now!
SDL_Surface* BustaGraphicsSprites::getCustomerSprites(ANIM_CUSTOMER_STATES state, int customerPortrait)
{
		return customer_sprites[customerPortrait][state];
}

SDL_Surface* BustaGraphicsSprites::getChefPortraits(BustaModelCharacter chef)
{
	return chef_portraits[chef];
}


SDL_Surface* BustaGraphicsSprites::getTipJarSprite(int percent_full)
{
		int stage = 100/TIP_JAR_STAGES;

		//(percent_full - 1) so that it is always < 100
		return tip_jar_sprites[(percent_full-1)/stage];
}

//Returns the correct food sprite based the food_piece passed in
SDL_Surface* BustaGraphicsSprites::getFoodSprite(BustaModelFoodPiece* food_piece)
{
		return food_sprites[(int) food_piece->getType()][(int) food_piece->getState()];

}

SDL_Surface* BustaGraphicsSprites::getSmallFoodSprite(BustaModelFoodPiece* food_piece)
{
		return food_sprites_small[(int) food_piece->getType()];

}

SDL_Surface* BustaGraphicsSprites::getCrusher(int state)
{
	return crusher_sprites[state%2];
}

