/*	Filename:		BustaModelCompleteSandwich.h
 *	Description:	Busta' Sandwich complete sandwich model.
 *
 */

#ifndef BUSTA_SANDWICH_COMPLETESANDWICH
#define BUSTA_SANDWICH_COMPLETESANDWICH

#include "BustaModelAnimation.h"
#include "BustaModelGameState.h"

/**
 * @class BustaModelCompleteSandwich
 * @brief Busta'Sandwich CompleteSandwich Model
 *
 * Busta'Sandwich Complete Sandwich model for representing sandwiches that have been made
 */
class BustaModelCompleteSandwich
{
	public:
		/**
		 * @brief Constructor
		 *
		 * Initializes variables
		 */
		BustaModelCompleteSandwich();
		
		/**
		 * @brief Destructor
		 *
		 * Deletes all of the foodpiece models within the complete sandwich represenation
		 *
		 */
		~BustaModelCompleteSandwich();


		/**
		 * @brief Gets opponent id
		 *
		 * Returns the opponent id that the sandwich is being sent to
		 */
		int getWho();

		/**
		 * @brief Sets opponent id
		 *
		 * Sets who the sandwich is going to
		 */
		void setWho(int w);

		/**
		 * @brief Get specified sandwich ingredient
		 *
		 * Returns the ingredient at the specified location in the array
		 */
		BustaModelFoodPiece* getIngredient(int n);

		/**
		 * @brief Sets specified sandwich ingredient
		 *
		 * Sets the ingredient at the specified location in the array to given foodpiece, deletes
		 * foodpiece currently in that location
		 */
		void setIngredient(BustaModelFoodPiece* fp, int n);

		/**
		 * @brief returns freshness state of complete sandwich
		 *
		 * Returns an int between -1 and 1 with 1 being the most fresh
		 * 
		 */
		int getState();

		void setState(int st_val);

	private:
		int who;
		int state;
		BustaModelFoodPiece* ingredients[WIDTH];

		// energy boost special sandwich dynamically created
		FoodType ssEnergyBoost[WIDTH];
	
};

#endif
