/*	Filename:		BustaModelCustomer.h
 *	Description:	Busta' Sandwich Customer model.
 *
 */

#ifndef BUSTA_SANDWICH_CUSTOMER
#define BUSTA_SANDWICH_CUSTOMER

#define WIDTH_2 5  //need to change this!
//#define CUSTOMERTIMER 60

#include "BustaModelFoodPiece.h"
#include "BustaModelAnimation.h"
#include <stdlib.h>


/**
 * @class BustaModelCustomer
 * @brief Customer model
 *
 * Model representing the customer.  Has time left on order as well as sandwich requested
 */
class BustaModelCustomer
{
	public:

		int portrait;
		/**
		 * @brief Constructor
		 *
		 * Initializes variables
		 */
		BustaModelCustomer();

		/**
		 * @brief Destructor
		 *
		 */
		~BustaModelCustomer();


		/**
		 * @brief Time left on order
		 *
		 * Gets the time left on the customer order
		 */
		int getTime();

		/**
		 * @brief Adds time to order
		 *
		 * Adds specified amount of time to time left on the order
		 */
		void addTime(int t);

		/**
		 * @brief Sets the amount of time left to complete the customer's order
		 *
		 * Sets the amount of time left to complete the customer's order
		 *
		 * @param timeLeft The amount of time to allow before the customer's order is considered unfulfilled
		 */
		void setTimeLeft(int timeLeft);

		/**
		 * @brief Returns the amount of time that was allotted to fulfilling the order
		 *
		 * Returns the amount of time that the customer allowed to have their order fulfilled.
		 *	This number is a raw count of seconds.
		 *
		 * @return The number of seconds to fulfill the customer's order
		 */
		int getAllottedTime();

		/**
		 * @brief Sets the total amount of time set to fulfill the order
		 *
		 * Sets the total amount of time set to fulfill the order
		 *
		 * @param newTime The total amount of time to give to the order
		 */
		void setAllottedTime(int newTime);
		
		/**
		 * @brief Current order
		 *
		 * Returns pointer to the current Customer order
		 */
		BustaModelFoodPiece** getOrder();

		/**
		 * @brief Foodpiece in order
		 *
		 * Returns pointer to nth food piece in order
		 */
		BustaModelFoodPiece* getOrder(int n);
		
		/**
		 * @brief Sets the current Customer order
		 *	
		 *	Currently a stub function
		 */
		void setOrder(BustaModelFoodPiece** o);


		void resetCustomer();


	private:
		time_t timedone;			// absolute time at which the order must be fulfilled
		time_t allottedTime;			// total timed allotted to fulfilling the order
		BustaModelFoodPiece* order[WIDTH_2];
		
		/*
		 * @brief Returns a random amount of time for the customer's timer
		 *
		 * Determines a random time to assign to the customer's timer.  Returns a time based
		 *	from 45 - 75 seconds.
		 *
		 * @return Returns an integer between 45 and 75
		 */
		int getRandomTime();

};

#endif