/*	Filename:		BustaModelFoodPiece.h
 *	Description:	Busta' Sandwich FoodPiece model.
 *
 */

#ifndef BUSTA_SANDWICH_FOODPIECE
#define BUSTA_SANDWICH_FOODPIECE
#include <time.h>
#include <stdlib.h>

#include "BustaModelAnimation.h"

#define FOODTIMER 45

/**
 *@enum BustaModelFoodType
 *@brief Type of food
 * 
 * Lists different kinds of food
 */
enum FoodType{TOMATO,ONION,LETTUCE,CHEESE,SALAMI,CHICKEN,TURKEY,HAM,TOFU,MAYO,MUSTARD,PICKLES, NUMFOODPIECES};


enum FoodState{FRESH,NORMAL,ROTTING,GONE};

/**
 * @class BustaModelFoodPiece
 * @brief Food Piece model
 *
 * Model representing food piece, includes type, freshness state, timer
 */
class BustaModelFoodPiece
{
	public:
		/**
		 * @brief Constructor
		 *
		 * Initializes variables
		 */
		BustaModelFoodPiece();

		/**
		 * @brief Constructor
		 *
		 * Initializes variables, sets type of food piece to t, 
		 * state equal to s
		 */
		BustaModelFoodPiece(FoodType t, FoodState s);
		
		
		
		/**
		 * @brief Constructor
		 *
		 * If seed value != 0 then its is a pseudo-random FoodPiece constructor,
		 * else it will always construct the same FoodPiece() as long as seed != 0  
		 */
		BustaModelFoodPiece(int seed);

		BustaModelFoodPiece(FoodType t, FoodState s, bool a);

		/**
		 * @brief Destructor
		 *
		 * Destructor
		 *
		 */
		~BustaModelFoodPiece();


		/**
		 * @brief FoodPiece type
		 *
		 * Gets the type of FoodPiece
		 */
		FoodType getType();

		/**
		 * @brief Sets FoodPiece type
		 *
		 * Sets the type of FoodPiece
		 */
		void setType(FoodType t);
		
		/**
		 * @brief Freshness state
		 *
		 * Returns the current FoodPiece freshness state
		 */
		FoodState getState();

		
		/**
		 * @brief Sets freshness state
		 *
		 * Sets the current FoodPiece freshness state
		 */
		void setState(FoodState s);

		/**
		 * @brief Gets immobility
		 *
		 * Returns boolean, saying whether the foodpiece can be swapped around the board or not
		 */
		bool getImmobile();

		/**
		 * @brief Sets immobility
		 *
		 * Sets whether the foodpiece can be swapped around the board
		 */
		void setImmobile(bool i);

		/**
		 * @brief Updates state of FoodPiece
		 *
		 * Updates the current state of the foodpiece based upon the current time.  Returns 1 if switched states,
		 * 0 if didn't switch states, and -1 if foodPiece has rotted away
		 */
		int updateState();

		/**
		 * @brief Updates state of FoodPiece
		 *
		 * Updates the current state of the foodpiece based upon the current time.  Returns 1 if switched states,
		 * 0 if didn't switch states, and -1 if foodPiece has rotted away
		 */
		int freshenState();

		/**
		 * @brief Updates state of FoodPiece
		 *
		 * Updates the current state of the foodpiece based upon the current time.  Returns 1 if switched states,
		 * 0 if didn't switch states, and -1 if foodPiece has rotted away
		 */
		int spoilState();




	private:
		FoodType type;
		FoodState state;
		bool immobile;
		time_t switchtime;

		/*
		 *	Moves the foodpiece to the next state
		 *
		 */
		void incrementState();

		/*
		 *	Moves the foodpiece to the previous state
		 *
		 */
		void decrementState();
		

};

#endif