#include "BustaModelGameState.h"


BustaModelGameState::BustaModelGameState(){
	score = 0;
	tips = new BustaModelTipJar();
	chef = new BustaModelChef();
	cursor = new BustaModelCursor();
	srand((unsigned)time(NULL));

	int t;
	int food_pieces [NUMFOODPIECES];
	for (int i = 0; i < NUMFOODPIECES; i++){ 
		food_pieces[i] = 0;
	}
	for(int i = 0; i < HEIGHT; i++){ 
		for(int j = 0; j<WIDTH; j++){
			t = rand()%NUMFOODPIECES;
			food_pieces[t] = food_pieces[t] + 1;
			board[i][j] = new BustaModelFoodPiece((FoodType)t, FRESH);	
		}	
	}
	//for (int i = 0; i < NUMFOODPIECES; i++){
	//	printf("initial number of %d foodpieces: %d\n", i, food_pieces[i]);
	//}
	for(int k = 0; k<NUMCUSTOMERS; k++){
		customers[k] = new BustaModelCustomer();
		BustaModelFoodPiece* cust_order [WIDTH];
		for (int i = 0; i < WIDTH; i++){
			int r = rand()%NUMFOODPIECES; //pick a random food type
			while (food_pieces[r] == 0){  //check if that food type is present on board, if not pick again
				//printf("r zero %d\n", r);
				r = rand()%NUMFOODPIECES;
			}
			//printf("r non zero %d\n", r);
			cust_order[i] = new BustaModelFoodPiece ((FoodType)r, FRESH); //set the i ingredient of customer order
																		  //to foodtype r
			food_pieces[r] = food_pieces[r] - 1;
		}
		customers[k]->setOrder(cust_order);
		//for (int i = 0; i < NUMFOODPIECES; i++){
		//printf("after customer %d number of %d foodpieces: %d\n", k, i, food_pieces[i]);
		//}
	}
}

BustaModelGameState::~BustaModelGameState(){
	delete tips;
	delete chef;
	delete cursor;

	for(int i =0; i<HEIGHT; i++){
		for(int j = 0; j<WIDTH; j++){
			delete board[i][j];
		}	
	}

	for(int k = 0; k<NUMCUSTOMERS; k++){
		delete customers[k];
	}
}


int BustaModelGameState::getScore(){
	return score;
}

void BustaModelGameState::setScore(int s){
	score = s;
}

void BustaModelGameState::addCustomer(BustaModelCustomer* c){
	if(customers[NUMCUSTOMERS-1]!= NULL)
	{
		delete customers[0];

		for(int i=0; i<NUMCUSTOMERS-1; i++){
			customers[i] = customers[i+1];
		}
	}

	customers[NUMCUSTOMERS-1] = c;
}

int BustaModelGameState::removeCustomer(BustaModelCustomer* c){
	int i = 0;

	while(customers[i]!=c && i<NUMCUSTOMERS){
		i++;
	}

	if(i==NUMCUSTOMERS){
		return -1;
	}
	else{
		customers[i]->resetCustomer();
	}
}


BustaModelCustomer*  BustaModelGameState::getCustomer(int n){
	return customers[n];
}


BustaModelTipJar* BustaModelGameState::getTipJar(){
	return tips;
}


BustaModelFoodPiece* BustaModelGameState::getBoard(int i,int j){
	return board[i][j];
}


BustaModelChef* BustaModelGameState::getChef(){
	return chef;
}

BustaModelCursor* BustaModelGameState::getCursor(){
	return cursor;
}


void BustaModelGameState::setBoard(int i,int j, BustaModelFoodPiece* fp){
	board[i][j] = fp;
}

void BustaModelGameState::removeFromBoard(int i, int j){
	delete board[i][j];
	board[i][j]=NULL;
}

void BustaModelGameState::updateFoodPieces(int u){
	int change;

	for(int i=HEIGHT-1; i>=0;i--){
		for(int j =0; j<WIDTH; j++){
			if(u==0){
				change = board[i][j]->updateState();
			}
			else if(u==1){
				change = board[i][j]->freshenState();
			}
			else if(u==-1){
				change = board[i][j]->spoilState();
			}
			else{
				printf("Invalid update choice!!!\n");
			}

			if(change==1){
				//printf("%d changing state", time(NULL));
			}

			if(change==-1){
				//printf("start delete\n");
				delete board[i][j];
				board[i][j]=NULL;

				//printf("done delete");

				for(int k=i;k>0;k--){
					board[k][j] = board[k-1][j];
				}

				int t = rand()%NUMFOODPIECES;
				board[0][j] = new BustaModelFoodPiece((FoodType)t, FRESH);
				j--;
			}
		}
	}
}


int BustaModelGameState::updateCustomers(){
	int t;
	int numRemoved = 0;
	int food_pieces [NUMFOODPIECES];
	for (int i = 0; i < NUMFOODPIECES; i++){ 
		food_pieces[i] = 0;
	}
	for (int row = 0; row < HEIGHT; row++){
		for (int col = 0; col < WIDTH; col++){
			int t = board[row][col]->getType();
			food_pieces[t] = food_pieces[t] + 1;
		}
	}
	BustaModelFoodPiece* cust_order [WIDTH];
		for (int i = 0; i < WIDTH; i++){
			int r = rand()%NUMFOODPIECES; //pick a random food type
			while (food_pieces[r] == 0){  //check if that food type is present on board, if not pick again
				//printf("r zero %d\n", r);
				r = rand()%NUMFOODPIECES;
			}
			//printf("r non zero %d\n", r);
			cust_order[i] = new BustaModelFoodPiece ((FoodType)r, FRESH); //set the i ingredient of customer order
																		  //to foodtype r
			food_pieces[r] = food_pieces[r] - 1;
		}
	for(int i=0; i<NUMCUSTOMERS; i++){
		t = customers[i]->getTime();

		if(t<=0){
			BustaModelCustomer* c = customers[i];
			c->resetCustomer();
			c->setOrder(cust_order);
			numRemoved++;
		}
	}

	return numRemoved;
}

void BustaModelGameState::jumbleBoard()
{
	for (int row = 0; row < HEIGHT; row++){
		for (int col = 0; col<WIDTH; col++){
			int swaprow = rand()%HEIGHT;
			int swapcol = rand()%WIDTH;

			swapTwo(row, col, swaprow, swapcol);

		}
	}	

}

void BustaModelGameState::swapTwo(BustaModelFoodPiece* first, BustaModelFoodPiece* second)
{
	BustaModelFoodPiece* swapTemp;		// temporary placeholder for swapping

	swapTemp = first;	// std 3-way swap
	first = second;
	second = swapTemp;
}

void BustaModelGameState::swapTwo(int firstX, int firstY, int secondX, int secondY)
{
	if ( ((firstX >= WIDTH) || (firstX < 0)) ||		// if the first x-coord is out of bounds
		((firstY >= HEIGHT) || (firstY < 0)) ||	// OR the first y-coord is out of bounds
		((secondX >= WIDTH) || (secondX < 0)) ||	// OR the second x-coord is out of bounds
		((secondY >= HEIGHT) || (secondY < 0))		// OR the second y-coord is out of bounds
		)
	{
		// invalid boundaries supplied, return
		return;
	}

	BustaModelFoodPiece* swapTemp;							// temporary swap space
	BustaModelFoodPiece* first = board[firstX][firstY];		// the first food piece
	BustaModelFoodPiece* second = board[secondX][secondY];	// second food piece

	swapTemp = first;	// std 3-way swap
	first = second;
	second = swapTemp;
}
