/*	Filename:		BustaModelMPGameState.h
 *	Description:	Busta' Sandwich Multi-player GameState model.
 *
 */

#define NULL 0

#ifndef BUSTA_SANDWICH_MPGAMESTATE
#define BUSTA_SANDWICH_MPGAMESTATE

#include "BustaModelOpponent.h"
#include "BustaModelGameState.h"

/**
 * @class BustaModelMPGameState
 * @brief Multi-player Game State representation
 * 
 * Game state representation for MP opponents.  Contains Game State model for selected opponent, plus important info for other opponents
 */
class BustaModelMPGameState
{
	friend class MPGameActionListener;


	public:
		/**
		 * @brief Constructor
		 *
		 * Initializes variables
		 */
		BustaModelMPGameState();

		/**
		 * @brief CONSTRUCTOR
		 *
		 * Initializes a full gamestate, and an array of opponentCount brief gamestates.
		 *
		 * @param opponentCount The number of brief gamestates to construct
		 */
		BustaModelMPGameState(int opponentCount);

		/**
		 * @brief Constructor
		 *
		 * Initializes variables
		 */
		BustaModelMPGameState(BustaModelGameState* gs, BustaModelOpponent** oppos, int n);


		/**
		 * @brief Destructor
		 * 
		 * Destructor
		 */
		~BustaModelMPGameState();

		/**
		 * @brief Returns the identification number of the fully viewed opponent
		 *
		 * Returns the network identification number of the opponent whose full gamestate is being
		 *	tracked.
		 *
		 * @return The network identification number of the fully viewed opponent.  Returns -1 if no
		 *	opponent is being fully viewed.
		 */
		int getFullViewID();

		/**
		 * @brief Gets opponent GameState
		 *
		 * Gets pointer to the GameState of highlighted opponent
		 */
		BustaModelGameState* getGameState();


		/**
		 * @brief Gets opponent
		 *
		 * Returns a pointer to the opponent in the nth location in the array
		 */
		BustaModelOpponent*  getOpponent(int n);

		
		/**
		 * @brief Returns opponent array
		 *
		 * Returns the entire array of opponent brief states
		 *
		 * @return A BustaModelOpponent pointer array representing the opponent brief game states
		 */
		BustaModelOpponent** getOpponents();

		/**
		 * @brief Returns the sequence number of the gamestate
		 *
		 * Returns the sequence number associated with the gamestate.  Each gamestate that is received from
		 *	the server is marked with a sequence number.  This sequence number is stored, and it is this
		 *	method that returns that sequence number.
		 *
		 * @return The sequence number of the gamestate
		 */
		int getSeqNo();

		/**
		 * @brief Sets the identification number of the opponent to be fully viewed
		 *
		 * Sets the network identification number of the opponent that is being fully viewed.  This
		 *	should be the identification number of the opponent whose gamestate is being tracked in
		 *	opponentGS.
		 *
		 * @param oID The identification number of the fully tracked opponent
		 */
		void setFullViewID(int oID);

		/**
		 * @brief Sets the number of tracked brief opponent states
		 *
		 * Sets the number of opponents to track using brief game states.
		 */
		void setNumOpps(int opponentCount);

		/**
		 * @brief Sets the sequence number of the gamestate
		 *
		 * Sets the sequence number associated with this gamestate to a particular value.
		 *
		 * @param newSeqNo The sequence number to set the gamestate to
		 *
		 * @see getSeqNo()
		 */
		void setSeqNo(int newSeqNo);

	private:
		int SeqNo;						// sequence number of the game state received from server
		int fullViewOpponentID;			// network identification number of the fully viewed opponent
		int numopps;
		BustaModelGameState* opponentGS;
		BustaModelOpponent** opponents;
};

#endif
