/*	Filename:		BustaModelOpponent.h
 *	Description:	Busta' Sandwich Opponent model.
 *
 */

#ifndef BUSTA_SANDWICH_OPPONENT
#define BUSTA_SANDWICH_OPPONENT

#include "BustaModelTipJar.h"
#include <string.h>

/**
 * @class BustaModelOpponent
 * @brief Opponent model
 * 
 *	Opponent representation, has character, health, and tip jar
 */
class BustaModelOpponent
{
	public:
		/**
		 * @brief Constructor
		 *
		 * Initializes variables
		 */
		BustaModelOpponent();

		/**
		 * @brief Constructor
		 *
		 * Initializes variables
		 */
		BustaModelOpponent(int h, int s, char* name);


		/**
		 * @brief Destructor
		 * 
		 *	Deletes tip jar
		 */
		~BustaModelOpponent();

		/**
		 * @brief Gets Character
		 *
		 * Returns int representing character ID
		 */
//		int getCharacter();

		/**
		 * @brief gets health of character
		 *
		 * Gets the health of the opponent
		 */
		int getHealth();

		/**
		 * @brief Returns the name of the opponent
		 *
		 * Returns the name of the opponent
		 *
		 * @return The name of the opponent
		 */
		char* getName();

		/**
		 * @brief gets amount in tipjar
		 *
		 * returns int with current amount in opponent tip jar
		 */
		int getTips();

		/**
		 * @brief Returns the tip jar
		 *
		 * Returns the tip jar associated with this opponent
		 *
		 * @return The tip jar of the opponent
		 */
		BustaModelTipJar* getTipJar();

		/**
		 * @brief Returns whether or not the opponent's data is valid
		 *
		 * Returns if the opponent's data contained is actually valid.  If it is not valid, the data does not
		 *	represent a real opponent or is no longer correct and should not be processed.
		 *
		 * @return Whether the data for this opponent is valid
		 */
		bool isValid();

		/**
		 * @brief Sets the character identification
		 *
		 * Sets the character identification
		 *
		 * @param cID The identification number to assign to the opponent
		 */
//		void setCharacter(int cID);

		/**
		 * @brief Sets the health of the opponent
		 *
		 * Sets the health of the opponent
		 *
		 * @param newHealth The amount of health to assign to the opponent
		 */
		void setHealth(int newHealth);

		/**
		 *
		 *
		 *
		 */
		void setPlayerName(char* newName);

		/**
		 * @brief Sets whether or not this opponent should be viewed
		 *
		 * Sets whether or not this opponent should be viewed.  If an opponent is not to be viewed, the
		 *	data contained within is irrelevant.  In the case of displaying graphics, this opponent's stats
		 *	need not even be displayed.
		 *
		 * @param valid Whether the opponent data is valid
		 */
		void setValid(bool valid);

	private:
		bool isvalid;				// marks whether or not the opponent should be tracked
		//int character;
		int health;
		BustaModelTipJar* tips;
		char* playerName;			// name of the player who is represented by the model
};

#endif