/*	Filename:		BustaModelTipJar.h
 *	Description:	Busta' Sandwich TipJar model.
 *
 */

#ifndef BUSTA_SANDWICH_TIPJAR
#define BUSTA_SANDWICH_TIPJAR


/**
 * @class BustaModelTipJar
 * @brief Tip Jar
 *
 * Tip jar contains tips from customers, has specific amount that it take to
 * fill, and cannot overflow
 *
 */
class BustaModelTipJar
{
	public:
		/**
		 * @brief Constructor
		 *
		 * Initializes variables
		 */
		BustaModelTipJar();

		/**
		 * @brief Constructor
		 *
		 * Initializes variables with fullamount s
		 */
		BustaModelTipJar(int s);


		/**
		 * @brief Destructor
		 *
		 * Destructor
		 */
		~BustaModelTipJar();

		/**
		 * @brief Gets size of tipjar
		 * 
		 * Gets the amount that can fit in the tip jar
		 */
		int getSize();

		/**
		 * @brief Sets size of tipjar
		 * 
		 * Sets the amount that can fit in the tip jar
		 */
		void setSize(int s);

		/**
		 * @brief Returns tips in jar
		 *
		 * returns the amount of tips in the tipjar
		 */
		int getTips();

		/**
		 * @brief adds specified amount to the tipjar; 
		 *
		 *	can only add tips up until full
		 */
		void addTips(int t);

		/**
		 * @brief Sets the amount of tips in the tip jar
		 *
		 * Sets the amount of tips in the tip jar.  If the new tip amount exceeds the size of the tip jar
		 *	the tip amount is set to the size of the tip jar.
		 *
		 * @param tipAmount The amount of tips to assign to the jar
		 */
		void setTips(int tipAmount);
		
		/**
		 * @brief removes tips from tipjar.  
		 *
		 * returns amount actually subtracted
		 */
		int subtractTips(int t);

		/**
		 * @brief Returns true if tipjar is full
		 *
		 * returns true if tipjar is full
		 */
		bool isfull();


	private:
		int tips;
		int fullamount;
};

#endif