/*	Filename:		BustaNetInternal.cpp
 *	Description:	Provides the implementation of all internal message classes between the
 *					network client and game engine.
 *
 */

#include "BustaNetInternal.h"

// --------------------------------------------------------------------------------------
// --------------------------- BustaNetInternalMsg (Base) -------------------------------
// --------------------------------------------------------------------------------------
int BustaNetInternalMsg::getType()
{
	return type;
}

void BustaNetInternalMsg::setType(int newType)
{
	type = newType;
}

// --------------------------------------------------------------------------------------
// ------------------------------- MPSpecialSandwich ------------------------------------
// --------------------------------------------------------------------------------------
MPSpecialSandwich::MPSpecialSandwich(int sendID, int receiveID, SpecialSandwich sSandwich)
{
	type = 301;
	senderID = sendID;
	receiverID = receiveID;
	specialSandwich = sSandwich;
}

int MPSpecialSandwich::getReceiverID()
{
	return receiverID;
}

int MPSpecialSandwich::getSenderID()
{
	return senderID;
}

SpecialSandwich MPSpecialSandwich::getSpecialSandwich()
{
	return specialSandwich;
}

// --------------------------------------------------------------------------------------
// -------------------------------- MPCharSelected --------------------------------------
// --------------------------------------------------------------------------------------
MPCharSelected::MPCharSelected(int pID, BustaModelCharacter selectedChar)
{
	type = 101;
	playerID = pID;
	character = selectedChar;
}

int MPCharSelected::getPlayerID()
{
	return playerID;
}

BustaModelCharacter MPCharSelected::getSelectedChar()
{
	return character;
}

// --------------------------------------------------------------------------------------
// ------------------------------ MPCharSelectCursor ------------------------------------
// --------------------------------------------------------------------------------------
MPCharSelectCursor::MPCharSelectCursor(int pID, int cursorX, int cursorY)
{
	type = 102;
	playerID = pID;
	playerCursor = new BustaModelPosition(cursorX, cursorY);
}

MPCharSelectCursor::~MPCharSelectCursor()
{
	delete playerCursor;
}

int MPCharSelectCursor::getPlayerID()
{
	return playerID;
}

BustaModelPosition* MPCharSelectCursor::getCursor()
{
	return playerCursor;
}

// --------------------------------------------------------------------------------------
// ------------------------------- MPCharSelectMode -------------------------------------
// --------------------------------------------------------------------------------------
MPCharSelectMode::MPCharSelectMode()
{
	type = 201;
}

// --------------------------------------------------------------------------------------
// --------------------------------- MPStartGame ----------------------------------------
// --------------------------------------------------------------------------------------
MPStartGame::MPStartGame()
{
	type = 202;
}

// --------------------------------------------------------------------------------------
// ---------------------------------- MPEndGame -----------------------------------------
// --------------------------------------------------------------------------------------
MPEndGame::MPEndGame(int playerCount)
{
	type = 203;
	Rankings = new int[playerCount];
	Points = new int[playerCount];
}

MPEndGame::MPEndGame(int playerCount, int* rankings, int* points)
{
	type = 203;
	Rankings = new int[playerCount];
	Points = new int[playerCount];
	for (int i = 0; i < playerCount; i++)
	{
		Rankings[i] = rankings[i];
		Points[i] = points[i];
	}
}

int* MPEndGame::getPoints()
{
	return Points;
}

int* MPEndGame::getRankings()
{
	return Rankings;
}

// --------------------------------------------------------------------------------------
// --------------------------------- MPPlayerLost ---------------------------------------
// --------------------------------------------------------------------------------------
MPPlayerLost::MPPlayerLost(int pID)
{
	type = 204;
	playerID = pID;
}

int MPPlayerLost::getPlayerID()
{
	return playerID;
}

// --------------------------------------------------------------------------------------
// ------------------------------- MPJoinResponse ---------------------------------------
// --------------------------------------------------------------------------------------
MPJoinResponse::MPJoinResponse(bool accept, char* msg, int cID)
{
	type = 1;
	acceptance = accept;
	reason = new char[101];
	strcpy(reason, (const char*)msg);
	clientID = cID;
}

bool MPJoinResponse::getAcceptance()
{
	return acceptance;
}

int MPJoinResponse::getClientID()
{
	return clientID;
}

char* MPJoinResponse::getReason()
{
	return reason;
}

// --------------------------------------------------------------------------------------
// ---------------------------------- MPNewPlayer ---------------------------------------
// --------------------------------------------------------------------------------------
MPNewPlayer::MPNewPlayer(int pID, char* pName)
{
	type = 2;
	playerID = pID;
	playerName = new char[21];
	strcpy(playerName, (const char*)pName);
}

int MPNewPlayer::getPlayerID()
{
	return playerID;
}

char* MPNewPlayer::getPlayerName()
{
	return playerName;
}

// --------------------------------------------------------------------------------------
// ------------------------------- MPPlayerDisconnect -----------------------------------
// --------------------------------------------------------------------------------------
MPPlayerDisconnect::MPPlayerDisconnect(int pID, char* msg)
{
	type = 3;
	playerID = pID;
	reason = new char[101];
	strcpy(reason, (const char*)msg);
}

int MPPlayerDisconnect::getPlayerID()
{
	return playerID;
}

char* MPPlayerDisconnect::getReason()
{
	return reason;
}

// --------------------------------------------------------------------------------------
// --------------------------------- MPServerSettings -----------------------------------
// --------------------------------------------------------------------------------------
MPServerSettings::MPServerSettings(char* sName, int nClients, netClient** sClients)
{
	type = 4;
	serverName = new char[21];
	strcpy(serverName, (const char*)sName);
	numClients = nClients;
	clients = sClients;
}

MPServerSettings::~MPServerSettings()
{
	// free memory allocated for each client
	for (int i = 0; i < numClients; i++)
	{
		free(clients[i]);
	}
}

netClient* MPServerSettings::getClient(int index)
{	
	if ( (index < 0) || (index > numClients) )
		return NULL;
	else
		return clients[index];
}

netClient** MPServerSettings::getClients()
{
	return clients;
}

int MPServerSettings::getNumClients()
{
	return numClients;
}

char* MPServerSettings::getServerName()
{
	return serverName;
}

void MPServerSettings::setServerName(char* newName)
{
	serverName = newName;
}

// --------------------------------------------------------------------------------------
// ------------------------------ MPPlayerReady -----------------------------------------
// --------------------------------------------------------------------------------------
MPPlayerReady::MPPlayerReady(int cID)
{
	type = 103;
	clientID = cID;
}

int MPPlayerReady::getClientID()
{
	return clientID;
}

// --------------------------------------------------------------------------------------
// -------------------------------- MPNoUpdate ------------------------------------------
// --------------------------------------------------------------------------------------
MPNoUpdate::MPNoUpdate()
{
	type = 600;
}

// --------------------------------------------------------------------------------------
// --------------------------------- MPError --------------------------------------------
// --------------------------------------------------------------------------------------
MPError::MPError()
{
	type = 666;
}
