/*	Filename:		BustaNetMsgQ.h
 *	Description:	Internal network message queue definition.
 *
 */

#ifndef BUSTASANDWICH_NETMSG_Q
#define BUSTASANDWICH_NETMSG_Q

#include "BustaNetMsgQNode.h"

/**
 * @class BustaNetMsgQ
 * @brief Network Message Queue
 *
 * This is a linked list implementation of a message queue.  The queue is provided by the client
 *	to the engine such that multiple server messages or special sandwiches can be sent before
 *	the engine addresses the messages.
 */
class BustaNetMsgQ
{
	public:
		/**
		 * @brief CONSTRUCTOR
		 *
		 * Creates an empty message queue.
		 */
		BustaNetMsgQ();

		/**
		 * @brief DESTRUCTOR
		 *
		 * Empties queue and frees memory.
		 */
		~BustaNetMsgQ();

		/**
		 * @brief Pop from the front of the queue
		 *
		 * Returns a pointer to the first message in the queue.  The message must be deleted after
		 *	use!
		 *
		 * @return A pointer to the first message in the queue.  Returns null if the queue is empty.
		 */
		BustaNetInternalMsg* pop();

		/**
		 * @brief Adds a new message to the queue
		 *
		 * Adds a preconstructed network message to the end of the message queue.
		 *
		 * @param newNsg The new message to be added to the queue
		 */
		void enqueue(BustaNetInternalMsg* newMsg);

		/**
		 * @brief Empties the Queue
		 *
		 * The message queue is emptied of all messages.  All messages are deleted from memory.
		 */
		void flushQ();

		/**
		 * @brief Returns the count of messages in the queue
		 *
		 * Returns the count of messages currently in the message queue
		 */
		int getNodeCount();

		/**
		 * @brief Returns whether or not the queue is empty
		 *
		 * Returns if the queue is empty of messages.
		 *
		 * @return Returns true if the queue is empty and false otherwise
		 */
		bool isEmpty();

	private:
		int nodeCount;					// count of how many nodes in queue
		BustaNetMsgQNode* headNode;		// first node of queue
		BustaNetMsgQNode* tailNode;		// last node of queue
};

#endif