/*	Filename:		BustaSandwichServer.cpp
 *	Description:	Main stand-alone server application for Busta' Sandwich
 *
 */

#include <stdio.h>
#include <string.h>

#include "BustaNetServer.h"

/* void Usage(char** argv)
 *
 *	Prints proper usage of the application (input parameters, etc)
 */
void Usage(char** argv)
{
	printf("Usage: %s [OPTION]\n", argv[0]);

	printf("-h\tHelp.\n");
	printf("-n name\tServer name.\n");
	printf("-s size\tMax server size (2-8).\n");
	printf("-v\tVerbose output.\n");
	printf("-d\tDebug output (Warning: prints lots of information).\n");
}

void printSafeToExit()
{
	printf("\nThe server is now empty.\n");
	printf("It is safe to exit the server until a client connects...\n");
	printf("Exit Server: (Ctrl-C)\n");
}

int init(BustaNetServer* server, char* serverName, int maxClients, bool debug, bool extreme_debug)
{
	server->setName(serverName);	
	server->setMaxClients(maxClients);
	server->setDebug(debug);
	server->setExtrememDebug(extreme_debug);

	char* initRet = server->init(23427);
	if(initRet != 0)
	{
		printf("Initialization error: %s", initRet);
		printf("Aborting\n");
		return 1;
	}
	return 0;
}

int main(int argc, char** argv)
{
	int AcceptRet, CharSelRet, GameRet;			// Return values of all server states
	char* initRet;								// Return value of server initialization
	char* serverName = "BustaSandwich Server";	// server's assigned name
	int maxClients = 8;							// server's max clients
	bool debug = false;							// debug output
	bool extreme_debug = false;					// an extra level of debug output

	// Create instance of server
	BustaNetServer* server = new BustaNetServer();


	// Parameters supplied
	if (argc > 1)
	{
		// Iterate across each parameter set server settings
		for (int i = 1; i < argc; i++)
		{
			if (strcmp(argv[i], "-h") == 0)							// Help Requested
			{
				Usage(argv);		// Print proper usage of applications
				return 0;
			}
			else if (strcmp(argv[i], "-v") == 0)					// Output Debug 
			{
				 debug = true;
			}
			else if (strcmp(argv[i], "-d") == 0)					// Output Debug 
			{
				 debug = true;
				 extreme_debug = true;
			}
			else if (strcmp(argv[i], "-n") == 0)					// Server Name
			{
				if (i == argc-1)
				{
					Usage(argv);	// Print proper usage of applications
					return 1;
				}

				serverName = argv[i++];
			}
			else if (strcmp(argv[i], "-s") == 0)					// Server Size
			{
				if (i == argc-1)
				{
					Usage(argv);	// Print proper usage of applications
					return 1;
				}

				sscanf(argv[++i],"%d",&maxClients);
				if(maxClients > 8)
				{
					maxClients = 8;
					printf("The maximum server size is 8.\n\n");
				}
			}
			else													// Invalid Parameter
			{
					printf("Invalid Parameter: %s\n", argv[i]);
					printf("Aborting\n");
					return 1;
			}
		}
	}

	// Initialize server
	if( init(server, serverName, maxClients, debug, extreme_debug) == 1 )
		return 1;

	// Output settings used to stdio
	system("cls");
	printf("+-----------------------------------+\n");
	printf("|      Busta' Sandwich Server       |\n");
	printf("+-----------------------------------+\n");
	printf("| Server Name: %20s |\n", server->getName());
	printf("| Maximum Players Allowed: %d        |\n", server->getMaxClients());
	printf("+-----------------------------------+\n\n");

	printSafeToExit();

	for (;;)
	{
		/* Accepting incoming client connections */
		AcceptRet = server->setStateAccept();

		// If error occurs, print to console and exit
		if (AcceptRet <= 0)
		{
			printf("Error Connecting Clients\n");
			printf("Aborting\n");
			return 1;
		}

		/* Character Selection State */
		CharSelRet = server->setStateCharSelect();

		// If error occurs, print to console and exit
		switch(CharSelRet)
		{
		case 1: break;
		case 0:
		case -1:
			printf("Error During Character Selection\n");
			printf("Aborting\n");
			return 1;
		case 2:
			delete server;
			server = new BustaNetServer();
			init(server, serverName, maxClients, debug, extreme_debug);
			printSafeToExit();
			continue;
		}


		/* Game Loop */
		GameRet = server->setStateGame();
		
		// If error occurs, print to console and exit
		if(GameRet <= 0)
		{
			printf("Error During Game Loop\n");
			printf("Aborting\n");
			return 1;
		}

		delete server;
		server = new BustaNetServer();
		init(server, serverName, maxClients, debug, extreme_debug);
		printSafeToExit();
	}

	// Free Allocated Memory
	delete(server);

	return 0;
}
